# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Classes and global objects related to U{XML Namespaces<http://www.w3.org/TR/2006/REC-xml-names-20060816/index.html>}.

Since namespaces hold all referenceable objects, this module also defines the
infrastructure for resolving named object references, such as schema
components.
"""

import pyxb
import pyxb.utils.utility
from pyxb.utils import six
import xml.dom
import logging

_log = logging.getLogger(__name__)

@pyxb.utils.utility.BackfillComparisons
class ExpandedName (pyxb.cscRoot):

    """Represent an U{expanded name
    <http://www.w3.org/TR/REC-xml-names/#dt-expname>}, which pairs a
    namespace with a local name.

    Because a large number of local elements, and most attributes, have no
    namespace associated with them, this is optimized for representing names
    with an absent namespace.  The hash and equality test methods are set so
    that a plain string is equivalent to a tuple of C{None} and that string.

    Note that absent namespaces can be represented in two ways: with a
    namespace of C{None} (the name "has no namespace"), and with a namespace
    that is an L{absent namespace <Namespace.CreateAbsentNamespace>} (the name
    "has an absent namespace").  Hash code calculations are done so that the
    two alternatives produce the same hash; however, comparison is done so
    that the two are distinguished.  The latter is the intended behavior; the
    former should not be counted upon.

    This class allows direct lookup of the named object within a category by
    using the category name as an accessor function.  That is, if the
    namespace of the expanded name C{en} has a category 'typeDefinition', then
    the following two expressions are equivalent::

      en.typeDefinition()
      en.namespace().categoryMap('typeDefinition').get(en.localName())

    This class descends from C{tuple} so that its values can be used as
    dictionary keys without concern for pointer equivalence.
    """
    def namespace (self):
        """The L{Namespace} part of the expanded name."""
        return self.__namespace
    __namespace = None

    def namespaceURI (self):
        """Return the URI of the namespace, or C{None} if the namespace is absent."""
        return self.__namespaceURI
    __namespaceURI = None

    def localName (self):
        """The local part of the expanded name."""
        return self.__localName
    __localName = None

    # Cached tuple representation
    __expandedName = None

    def validateComponentModel (self):
        """Pass model validation through to namespace part."""
        return self.namespace().validateComponentModel()

    def uriTuple (self):
        """Return a tuple consisting of the namespace URI and the local name.

        This presents the expanded name as base Python types for persistent
        storage.  Be aware, though, that it will lose the association of the
        name with an absent namespace, if that matters to you."""
        return ( self.__namespaceURI, self.__localName )

    # Treat unrecognized attributes as potential accessor functions
    def __getattr__ (self, name):
        # Don't try to recognize private names (like __setstate__)
        if name.startswith('__'):
            return super(ExpandedName, self).__getattr__(name)
        ns = self.namespace()
        if ns is None:
            return lambda: None
        # Anything we're going to look stuff up in requires a component model.
        # Make sure we have one loaded.
        ns.validateComponentModel()
        # NOTE: This will raise pyxb.NamespaceError if the category does not exist.
        category_value = ns.categoryMap(name).get(self.localName())
        return lambda : category_value

    def createName (self, local_name):
        """Return a new expanded name in the namespace of this name.

        @param local_name: The local name portion of an expanded name.
        @return: An instance of L{ExpandedName}.
        """
        return ExpandedName(self.namespace(), local_name)

    def adoptName (self, name):
        """Return the input name, except if the input name has no namespace,
        return a name that uses the namespace from this name with the local
        name from the input name.

        Use this when the XML document has an unqualified name and we're
        processing using an absent default namespace.

        @warning: Be careful when using a global name to adopt a name from a
        local element: if the local element (with no namespace) has the same
        localName as but is different from the global element (with a
        namespace), this will improperly provide a namespace when one should
        not be present.  See the comments in
        L{pyxb.binding.basis.element.elementForName}.
        """

        if not isinstance(name, ExpandedName):
            name = ExpandedName(name)
        if name.namespace() is None:
            name = self.createName(name.localName())
        return name

    def __init__ (self, *args, **kw):
        """Create an expanded name.

        Expected argument patterns are:

         - ( C{str} ) : the local name in an absent namespace
         - ( L{ExpandedName} ) : a copy of the given expanded name
         - ( C{xml.dom.Node} ) : The name extracted from node.namespaceURI and node.localName
         - ( C{str}, C{str} ) : the namespace URI and the local name
         - ( L{Namespace}, C{str} ) : the namespace and the local name
         - ( L{ExpandedName}, C{str}) : the namespace from the expanded name, and the local name

        Wherever C{str} occurs C{unicode} is also permitted.

        @keyword fallback_namespace: Optional Namespace instance to use if the
        namespace would otherwise be None.  This is only used if it is an
        absent namespace.

        """
        fallback_namespace = kw.get('fallback_namespace')
        if 0 == len(args):
            raise pyxb.LogicError('Too few arguments to ExpandedName constructor')
        if 2 < len(args):
            raise pyxb.LogicError('Too many arguments to ExpandedName constructor')
        if 2 == len(args):
            # Namespace(str, unicode, Namespace) and local name basestring
            ( ns, ln ) = args
        else:
            # Local name basestring or ExpandedName or Node
            assert 1 == len(args)
            ln = args[0]
            ns = None
            if isinstance(ln, six.string_types):
                pass
            elif isinstance(ln, tuple) and (2 == len(ln)):
                (ns, ln) = ln
            elif isinstance(ln, ExpandedName):
                ns = ln.namespace()
                ln = ln.localName()
            elif isinstance(ln, xml.dom.Node):
                if not(ln.nodeType in (xml.dom.Node.ELEMENT_NODE, xml.dom.Node.ATTRIBUTE_NODE)):
                    raise pyxb.LogicError('Cannot create expanded name from non-element DOM node %s' % (ln.nodeType,))
                ns = ln.namespaceURI
                ln = ln.localName
            else:
                raise pyxb.LogicError('Unrecognized argument type %s' % (type(ln),))
        if (ns is None) and (fallback_namespace is not None):
            if fallback_namespace.isAbsentNamespace():
                ns = fallback_namespace
        if isinstance(ns, six.string_types):
            ns = NamespaceForURI(ns, create_if_missing=True)
        if isinstance(ns, ExpandedName):
            ns = ns.namespace()
        if (ns is not None) and not isinstance(ns, Namespace):
            raise pyxb.LogicError('ExpandedName must include a valid (perhaps absent) namespace, or None.')
        self.__namespace = ns
        if self.__namespace is not None:
            self.__namespaceURI = self.__namespace.uri()
        self.__localName = ln
        assert self.__localName is not None
        self.__expandedName = ( self.__namespace, self.__localName )
        self.__uriTuple = ( self.__namespaceURI, self.__localName )
        super(ExpandedName, self).__init__(*args, **kw)

    def __str__ (self):
        assert self.__localName is not None
        if self.__namespaceURI is not None:
            return '{%s}%s' % (self.__namespaceURI, self.__localName)
        return self.localName()

    def __hash__ (self):
        if self.__namespaceURI is None:
            # Handle both str and unicode hashes
            return type(self.__localName).__hash__(self.__localName)
        return tuple.__hash__(self.__expandedName)

    def __otherForCompare (self, other):
        if isinstance(other, six.string_types):
            other = ( None, other )
        if not isinstance(other, tuple):
            other = other.__uriTuple
        if isinstance(other[0], Namespace):
            other = ( other[0].uri(), other[1] )
        return other

    def __eq__ (self, other):
        if other is None:
            return False
        return 0 == pyxb.utils.utility.IteratedCompareMixed(self.__uriTuple, self.__otherForCompare(other))

    def __lt__ (self, other):
        if other is None:
            return False
        return 0 > pyxb.utils.utility.IteratedCompareMixed(self.__uriTuple, self.__otherForCompare(other))

    def getAttribute (self, dom_node):
        """Return the value of the attribute identified by this name in the given node.

        @return: An instance of C{xml.dom.Attr}, or C{None} if the node does
        not have an attribute with this name.
        """
        if dom_node.hasAttributeNS(self.__namespaceURI, self.__localName):
            return dom_node.getAttributeNS(self.__namespaceURI, self.__localName)
        return None

    def nodeMatches (self, dom_node):
        """Return C{True} iff the dom node expanded name matches this expanded name."""
        return (dom_node.localName == self.__localName) and (dom_node.namespaceURI == self.__namespaceURI)

class NamedObjectMap (dict):
    """An extended dictionary intended to assist with QName resolution.

    These dictionaries have an attribute that identifies a category of named
    objects within a Namespace; the specifications for various documents
    require that certain groups of objects must be unique, while uniqueness is
    not required between groups.  The dictionary also retains a pointer to the
    Namespace instance for which it holds objects."""
    def namespace (self):
        """The namespace to which the object map belongs."""
        return self.__namespace
    __namespace = None

    def category (self):
        """The category of objects (e.g., typeDefinition, elementDeclaration)."""
        return self.__category
    __category = None

    def __init__ (self, category, namespace, *args, **kw):
        self.__category = category
        self.__namespace = namespace
        super(NamedObjectMap, self).__init__(*args, **kw)

class _NamespaceCategory_mixin (pyxb.cscRoot):
    """Mix-in that aggregates those aspects of XMLNamespaces that hold
    references to categories of named objects.

    Arbitrary groups of named objects, each requiring unique names within
    themselves, can be saved.  Unless configured otherwise, the Namespace
    instance is extended with accessors that provide direct access to
    individual category maps.  The name of the method is the category name
    with a suffix of "s"; e.g., if a category "typeDefinition" exists, it can
    be accessed from the namespace using the syntax C{ns.typeDefinitions()}.

    Note that the returned value from the accessor is a live reference to
    the category map; changes made to the map are reflected in the
    namespace.
    """

    # Map from category strings to NamedObjectMap instances that
    # contain the dictionary for that category.
    __categoryMap = None

    def _reset (self):
        """CSC extension to reset fields of a Namespace.

        This one handles category-related data."""
        getattr(super(_NamespaceCategory_mixin, self), '_reset', lambda *args, **kw: None)()
        self.__categoryMap = { }

    def categories (self):
        """The list of individual categories held in this namespace."""
        return list(self.__categoryMap.keys())

    def _categoryMap (self):
        """Return the whole map from categories to named objects."""
        return self.__categoryMap

    def categoryMap (self, category):
        """Map from local names to NamedObjectMap instances for the given category."""
        try:
            return self.__categoryMap[category]
        except KeyError:
            raise pyxb.NamespaceError(self, '%s has no category %s' % (self, category))

    def __defineCategoryAccessors (self):
        """Define public methods on the Namespace which provide access to
        individual NamedObjectMaps based on their category.

        """
        for category in self.categories():
            accessor_name = category + 's'
            setattr(self, accessor_name, lambda _map=self.categoryMap(category): _map)

    def configureCategories (self, categories):
        """Ensure there is a map for each of the given categories.

        Category configuration
        L{activates<archive._NamespaceArchivable_mixin.isActive>} a namespace.

        Existing maps are not affected."""

        self._activate()
        if self.__categoryMap is None:
            self.__categoryMap = { }
        for category in categories:
            if not (category in self.__categoryMap):
                self.__categoryMap[category] = NamedObjectMap(category, self)
        self.__defineCategoryAccessors()
        return self

    def addCategoryObject (self, category, local_name, named_object):
        """Allow access to the named_object by looking up the local_name in
        the given category.

        Raises pyxb.NamespaceUniquenessError if an object with the same name
        already exists in the category."""
        name_map = self.categoryMap(category)
        old_object = name_map.get(local_name)
        if (old_object is not None) and (old_object != named_object):
            raise pyxb.NamespaceUniquenessError(self, '%s: name %s used for multiple values in %s' % (self, local_name, category))
        name_map[local_name] = named_object
        return named_object

    def replaceCategoryObject (self, category, local_name, old_object, new_object):
        """Replace the referenced object in the category.

        The new object will be added only if the old_object matches the
        current entry for local_name in the category."""
        name_map = self.categoryMap(category)
        if old_object == name_map.get(local_name):
            name_map[local_name] = new_object
        return name_map[local_name]

    def _replaceComponent_csc (self, existing_def, replacement_def):
        """Replace a component definition where present in the category maps.

        @note: This is a high-cost operation, as every item in every category
        map must be examined to see whether its value field matches
        C{existing_def}."""
        for (cat, registry) in six.iteritems(self.__categoryMap):
            for (k, v) in registry.items(): # NB: Not iteritems
                if v == existing_def:
                    del registry[k]
                    if replacement_def is not None:
                        registry[k] = replacement_def
        return getattr(super(_NamespaceCategory_mixin, self), '_replaceComponent_csc', lambda *args, **kw: replacement_def)(existing_def, replacement_def)

    # Verify that the namespace category map has no components recorded.  This
    # is the state that should hold prior to loading a saved namespace; at
    # tthe moment, we do not support aggregating components defined separately
    # into the same namespace.  That should be done at the schema level using
    # the "include" element.
    def __checkCategoriesEmpty (self):
        if self.__categoryMap is None:
            return True
        assert isinstance(self.__categoryMap, dict)
        if 0 == len(self.__categoryMap):
            return True
        for k in self.categories():
            if 0 < len(self.categoryMap(k)):
                return False
        return True

    def _namedObjects (self):
        objects = set()
        for category_map in six.itervalues(self.__categoryMap):
            objects.update(six.itervalues(category_map))
        return objects

    def _loadNamedObjects (self, category_map):
        """Add the named objects from the given map into the set held by this namespace.
        It is an error to name something which is already present."""
        self.configureCategories(six.iterkeys(category_map))
        for category in six.iterkeys(category_map):
            current_map = self.categoryMap(category)
            new_map = category_map[category]
            for (local_name, component) in six.iteritems(new_map):
                existing_component = current_map.get(local_name)
                if existing_component is None:
                    current_map[local_name] = component
                elif existing_component._allowUpdateFromOther(component):
                    existing_component._updateFromOther(component)
                else:
                    raise pyxb.NamespaceError(self, 'Load attempted to override %s %s in %s' % (category, local_name, self.uri()))
        self.__defineCategoryAccessors()

    def hasSchemaComponents (self):
        """Return C{True} iff schema components have been associated with this namespace.

        This only checks whether the corresponding categories have been added,
        not whether there are any entries in those categories.  It is useful
        for identifying namespaces that were incorporated through a
        declaration but never actually referenced."""
        return 'typeDefinition' in self.__categoryMap

    def _associateOrigins (self, module_record):
        """Add links from L{pyxb.namespace.archive._ObjectOrigin} instances.

        For any resolvable item in this namespace from an origin managed by
        the module_record, ensure that item can be found via a lookup through
        that origin.

        This allows these items to be found when a single namespace comprises
        items translated from different schema at different times using
        archives to maintain consistency."""
        assert module_record.namespace() == self
        module_record.resetCategoryObjects()
        self.configureCategories([archive.NamespaceArchive._AnonymousCategory()])
        origin_set = module_record.origins()
        for (cat, cat_map) in six.iteritems(self.__categoryMap):
            for (n, v) in six.iteritems(cat_map):
                if isinstance(v, archive._ArchivableObject_mixin) and (v._objectOrigin() in origin_set):
                    v._objectOrigin().addCategoryMember(cat, n, v)

class _ComponentDependency_mixin (pyxb.utils.utility.PrivateTransient_mixin, pyxb.cscRoot):
    """Mix-in for components that can depend on other components."""

    __PrivateTransient = set()

    # Cached frozenset of components on which this component depends.
    __bindingRequires = None
    __PrivateTransient.add('bindingRequires')

    def _resetClone_csc (self, **kw):
        """CSC extension to reset fields of a component.  This one clears
        dependency-related data, since the clone will have to revise its
        dependencies.
        @rtype: C{None}"""
        getattr(super(_ComponentDependency_mixin, self), '_resetClone_csc', lambda *_args, **_kw: None)(**kw)
        self.__bindingRequires = None

    def bindingRequires (self, reset=False, include_lax=False):
        """Return a set of components upon whose bindings this component's
        bindings depend.

        For example, bindings that are extensions or restrictions depend on
        their base types.  Complex type definition bindings require that the
        types of their attribute declarations be available at the class
        definition, and the types of their element declarations in the
        postscript.

        @keyword include_lax: if C{False} (default), only the requirements of
        the class itself are returned.  If C{True}, all requirements are
        returned.
        @rtype: C{set(L{pyxb.xmlschema.structures._SchemaComponent_mixin})}
        """
        if reset or (self.__bindingRequires is None):
            if isinstance(self, resolution._Resolvable_mixin) and not (self.isResolved()):
                raise pyxb.LogicError('Unresolved %s in %s: %s' % (self.__class__.__name__, self._namespaceContext().targetNamespace(), self.name()))
            self.__bindingRequires = self._bindingRequires_vx(include_lax)
        return self.__bindingRequires

    def _bindingRequires_vx (self, include_lax):
        """Placeholder for subclass method that identifies the necessary components.

        @note: Override in subclasses.

        @return: The component instances on which this component depends
        @rtype: C{frozenset}
        @raise LogicError: A subclass failed to implement this method
        """
        raise pyxb.LogicError('%s does not implement _bindingRequires_vx' % (type(self),))

class _NamespaceComponentAssociation_mixin (pyxb.cscRoot):
    """Mix-in for managing components defined within this namespace.

    The component set includes not only top-level named components (such as
    those accessible through category maps), but internal anonymous
    components, such as those involved in representing the content model of a
    complex type definition.  We need to be able to get a list of these
    components, sorted in dependency order, so that generated bindings do not
    attempt to refer to a binding that has not yet been generated."""

    # A set containing all components, named or unnamed, that belong to this
    # namespace.
    __components = None

    def _reset (self):
        """CSC extension to reset fields of a Namespace.

        This one handles data related to component association with a
        namespace."""
        getattr(super(_NamespaceComponentAssociation_mixin, self), '_reset', lambda *args, **kw: None)()
        self.__components = set()
        self.__origins = set()
        self.__schemaMap = { }

    def _associateComponent (self, component):
        """Record that the responsibility for the component belongs to this namespace."""
        self._activate()
        assert self.__components is not None
        assert isinstance(component, _ComponentDependency_mixin)
        assert component not in self.__components
        self.__components.add(component)

    def _replaceComponent_csc (self, existing_def, replacement_def):
        """Replace a component definition in the set of associated components.

        @raise KeyError: C{existing_def} is not in the set of components."""

        self.__components.remove(existing_def)
        if replacement_def is not None:
            self.__components.add(replacement_def)
        return getattr(super(_NamespaceComponentAssociation_mixin, self), '_replaceComponent_csc', lambda *args, **kw: replacement_def)(existing_def, replacement_def)

    def addSchema (self, schema):
        for sr in self.__origins:
            if isinstance(sr, archive._SchemaOrigin) and sr.match(schema=schema):
                _log.info('Hash for %s matches %s already registered as %s', schema.location(), sr.schema().location(), self)
                raise pyxb.SchemaUniquenessError(self, schema.location(), sr.schema())
        sr = archive._SchemaOrigin(schema=schema)
        schema.generationUID().associateObject(sr)
        self.__origins.add(sr)
        return sr

    def lookupSchemaByLocation (self, schema_location):
        for sr in self.__origins:
            if isinstance(sr, archive._SchemaOrigin) and sr.match(location=schema_location):
                return (True, sr.schema())
        for mr in self.moduleRecords():
            if mr.hasMatchingOrigin(location=schema_location):
                return (True, None)
        return (False, None)

    def schemas (self):
        s = set()
        for sr in self.__origins:
            if isinstance(sr, archive._SchemaOrigin) and (sr.schema() is not None):
                s.add(sr.schema())
        return s

    __origins = None

    def components (self):
        """Return a frozenset of all components, named or unnamed, belonging
        to this namespace."""
        return frozenset(self.__components)

    def _releaseNamespaceContexts (self):
        for c in self.__components:
            c._clearNamespaceContext()

from pyxb.namespace import archive
from pyxb.namespace.utility import NamespaceInstance
from pyxb.namespace.utility import NamespaceForURI
from pyxb.namespace.utility import CreateAbsentNamespace
from pyxb.namespace.utility import AvailableNamespaces
from pyxb.namespace import resolution
NamespaceContext = resolution.NamespaceContext

class Namespace (_NamespaceCategory_mixin, resolution._NamespaceResolution_mixin, _NamespaceComponentAssociation_mixin, archive._NamespaceArchivable_mixin):
    """Represents an XML namespace (a URI).

    There is at most one L{Namespace} class instance per namespace (URI).  The
    instance also supports associating arbitrary L{maps<NamedObjectMap>} from
    names to objects, in separate categories.  The default categories are
    configured externally; for example, the
    L{Schema<pyxb.xmlschema.structures.Schema>} component defines a category
    for each named component in XMLSchema, and the customizing subclass for
    WSDL definitions adds categories for the service bindings, messages, etc.

    Namespaces can be written to and loaded from pickled files.  See
    L{NamespaceArchive} for information.
    """

    # The URI for the namespace.  If the URI is None, this is an absent
    # namespace.
    __uri = None

    # An identifier, unique within a program using PyXB, used to distinguish
    # absent namespaces.  Currently this value is not accessible to the user,
    # and exists solely to provide a unique identifier when printing the
    # namespace as a string.  The class variable is used as a one-up counter,
    # which is assigned to the instance variable when an absent namespace
    # instance is created.
    __absentNamespaceID = 0

    # A prefix bound to this namespace by standard.  Current set known are applies to
    # xml and xmlns.
    __boundPrefix = None

    # A prefix set as a preferred prefix, generally by processing a namespace
    # declaration.
    __prefix = None

    # A map from URIs to Namespace instances.  Namespaces instances
    # must be unique for their URI.  See __new__().
    __Registry = { }

    # A set of all absent namespaces created.
    __AbsentNamespaces = set()

    # Optional description of the namespace
    __description = None

    # Indicates whether this namespace is built-in to the system
    __isBuiltinNamespace = False

    # Indicates whether this namespace is undeclared (available always)
    __isUndeclaredNamespace = False

    # Indicates whether this namespace was loaded from an archive
    __isLoadedNamespace = False

    # Archive from which the namespace can be read, or None if no archive
    # defines this namespace.
    __namespaceArchive = None

    # Indicates whether this namespace has been written to an archive
    __hasBeenArchived = False

    # Holds the module path for builtin modules until we get a ModuleRecord to
    # store that in.
    __builtinModulePath = None

    # A set of options defining how the Python bindings for this namespace
    # were generated.  Not currently used, since we don't have different
    # binding configurations yet.
    __bindingConfiguration = None

    # The namespace to use as the default namespace when constructing the
    # The namespace context used when creating built-in components that belong
    # to this namespace.  This is used to satisfy the low-level requirement
    # that all schema components have a namespace context; normally, that
    # context is built dynamically from the schema element.
    __initialNamespaceContext = None

    # The default_namespace parameter when creating the initial namespace
    # context.  Only used with built-in namespaces.
    __contextDefaultNamespace = None

    # The map from prefixes to namespaces as defined by the schema element for
    # this namespace.  Only used with built-in namespaces.
    __contextInScopeNamespaces = None

    @classmethod
    def _NamespaceForURI (cls, uri):
        """If a Namespace instance for the given URI exists, return it; otherwise return None.

        Note: Absent namespaces are not stored in the registry.  If you use
        one (e.g., for a schema with no target namespace), don't lose hold of
        it."""
        if uri is None:
            raise pyxb.UsageError('Absent namespaces are unlocatable')
        return cls.__Registry.get(uri)

    # A map from string UUIDs to absent Namespace instances.  Used for
    # in-session deserialization as required for cloning objects.  Non-absent
    # namespaces are identified by URI and recorded in __Registry.
    __AbsentNamespaceRegistry = { }

    # The UUID used to serialize this namespace. This serves the same role in
    # __AbsentNamespaceRegistry as the namespace URI does in __Registry, but
    # is retained only within a single PyXB session.
    __absentSerializedUUID = None

    __SerializedVariantAbsent = 'absent'

    def __getnewargs__ (self):
        """Pickling support.

        To ensure that unpickled Namespace instances are unique per
        URI, we ensure that the routine that creates unpickled
        instances knows what it's supposed to return."""
        if self.uri() is None:
            # We can't reconstruct absent namespaces.  However, it is
            # convenient to be able to use Python's copy module to clone
            # instances.  Support for that does require ability to identify
            # specific absent namespaces, which we do by representing them as
            # a tuple containing a variant tag and unique identifier.
            if self.__absentSerializedUUID is None:
                _log.warning('Instances with absent namespaces can only be reconstructed in-session')
                self.__absentSerializedUUID = pyxb.utils.utility.UniqueIdentifier()
                self.__AbsentNamespaceRegistry[self.__absentSerializedUUID.uid()] = self
            return ((self.__SerializedVariantAbsent, self.__absentSerializedUUID.uid()),)
        return (self.uri(),)

    def __new__ (cls, *args, **kw):
        """Pickling and singleton support.

        This ensures that no more than one Namespace instance exists
        for any given URI.  We could do this up in __init__, but that
        doesn't normally get called when unpickling instances; this
        does.  See also __getnewargs__()."""
        (uri,) = args
        if isinstance(uri, tuple):
            # Special handling to reconstruct absent namespaces.
            (variant, uid) = uri
            if cls.__SerializedVariantAbsent == variant:
                ns = cls.__AbsentNamespaceRegistry.get(uid)
                if ns is None:
                    raise pyxb.UsageError('Unable to reconstruct instance of absent namespace')
                return ns
            raise pyxb.LogicError('Unrecognized serialized namespace variant %s uid %s' % (variant, uid))
        elif not (uri in cls.__Registry):
            instance = object.__new__(cls)
            # Do this one step of __init__ so we can do checks during unpickling
            instance.__uri = uri
            instance._reset()
            # Absent namespaces are not stored in the registry.
            if uri is None:
                cls.__AbsentNamespaces.add(instance)
                return instance
            cls.__Registry[uri] = instance
        return cls.__Registry[uri]

    @classmethod
    def AvailableNamespaces (cls):
        """Return a set of all Namespace instances defined so far."""
        return cls.__AbsentNamespaces.union(six.itervalues(cls.__Registry))

    def __init__ (self, uri,
                  description=None,
                  builtin_namespace=None,
                  builtin_module_path=None,
                  is_undeclared_namespace=False,
                  is_loaded_namespace=False,
                  bound_prefix=None,
                  default_namespace=None,
                  in_scope_namespaces=None):
        """Create a new Namespace.

        The URI must be non-None, and must not already be assigned to
        a Namespace instance.  See _NamespaceForURI().

        User-created Namespace instances may also provide a description.

        Users should never provide a builtin_namespace parameter.
        """

        # New-style superclass invocation
        super(Namespace, self).__init__()

        self.__contextDefaultNamespace = default_namespace
        self.__contextInScopeNamespaces = in_scope_namespaces

        # Make sure that we're not trying to do something restricted to
        # built-in namespaces
        is_builtin_namespace = not (builtin_namespace is None)
        if not is_builtin_namespace:
            if bound_prefix is not None:
                raise pyxb.LogicError('Only permanent Namespaces may have bound prefixes')

        # We actually set the uri when this instance was allocated;
        # see __new__().
        assert self.__uri == uri
        self.__boundPrefix = bound_prefix
        self.__description = description
        self.__isBuiltinNamespace = is_builtin_namespace
        self.__builtinNamespaceVariable = builtin_namespace
        self.__builtinModulePath = builtin_module_path
        self.__isUndeclaredNamespace = is_undeclared_namespace
        self.__isLoadedNamespace = is_loaded_namespace

        self._reset()

        assert (self.__uri is None) or (self.__Registry[self.__uri] == self)

    def _reset (self):
        assert not self.isActive()
        getattr(super(Namespace, self), '_reset', lambda *args, **kw: None)()
        self.__initialNamespaceContext = None

    def uri (self):
        """Return the URI for the namespace represented by this instance.

        If the URI is None, this is an absent namespace, used to hold
        declarations not associated with a namespace (e.g., from schema with
        no target namespace)."""
        return self.__uri

    def setPrefix (self, prefix):
        if self.__boundPrefix is not None:
            if self.__boundPrefix == prefix:
                return self
            raise pyxb.NamespaceError(self, 'Cannot change the prefix of a bound namespace')
        self.__prefix = prefix
        return self

    def prefix (self):
        if self.__boundPrefix:
            return self.__boundPrefix
        return self.__prefix

    def isAbsentNamespace (self):
        """Return True iff this namespace is an absent namespace.

        Absent namespaces have no namespace URI; they exist only to
        hold components created from schemas with no target
        namespace."""
        return self.__uri is None

    def fallbackNamespace (self):
        """When known to be operating in this namespace, provide the Namespace
        instance to be used when names are associated with no namespace."""
        if self.isAbsentNamespace():
            return self
        return None

    @classmethod
    def CreateAbsentNamespace (cls):
        """Create an absent namespace.

        Use this instead of the standard constructor, in case we need
        to augment it with a uuid or the like."""
        rv = Namespace(None)
        rv.__absentNamespaceID = cls.__absentNamespaceID
        cls.__absentNamespaceID += 1

        return rv

    def _overrideAbsentNamespace (self, uri):
        assert self.isAbsentNamespace()
        self.__uri = uri

    def boundPrefix (self):
        """Return the standard prefix to be used for this namespace.

        Only a few namespace prefixes are bound to namespaces: xml and xmlns
        are two.  In all other cases, this method should return None.  The
        infrastructure attempts to prevent user creation of Namespace
        instances that have bound prefixes."""
        return self.__boundPrefix

    def isBuiltinNamespace (self):
        """Return True iff this namespace was defined by the infrastructure.

        That is the case for all namespaces in the Namespace module."""
        return self.__isBuiltinNamespace

    def builtinNamespaceRepresentation (self):
        assert self.__builtinNamespaceVariable is not None
        return 'pyxb.namespace.%s' % (self.__builtinNamespaceVariable,)

    def builtinModulePath (self):
        from pyxb.namespace import builtin
        if not self.__builtinModulePath:
            raise pyxb.LogicError('Namespace has no built-in module: %s' % (self,))
        mr = self.lookupModuleRecordByUID(builtin.BuiltInObjectUID)
        assert mr is not None
        assert mr.modulePath() == self.__builtinModulePath
        return self.__builtinModulePath

    def isUndeclaredNamespace (self):
        """Return True iff this namespace is always available
        regardless of whether there is a declaration for it.

        This is the case only for the
        xml(http://www.w3.org/XML/1998/namespace) and
        xmlns(http://www.w3.org/2000/xmlns/) namespaces."""
        return self.__isUndeclaredNamespace

    def isLoadedNamespace (self):
        """Return C{True} iff this namespace was loaded from a namespace archive."""
        return self.__isLoadedNamespace

    def hasBeenArchived (self):
        """Return C{True} iff this namespace has been saved to a namespace archive.
        See also L{isLoadedNamespace}."""
        return self.__hasBeenArchived

    def description (self, description=None):
        """Get, or set, a textual description of the namespace."""
        if description is not None:
            self.__description = description
        return self.__description

    def nodeIsNamed (self, node, *local_names):
        return (node.namespaceURI == self.uri()) and (node.localName in local_names)

    def createExpandedName (self, local_name):
        return ExpandedName(self, local_name)

    def __getstate__ (self):
        """Support pickling.

        Well, no, not really.  Because namespace instances must be unique, we
        represent them as their URI, and that's done by __getnewargs__
        above.  All the interesting information is in the ModuleRecords."""
        return {}

    def _defineBuiltins_ox (self, structures_module):
        pass

    __definedBuiltins = False
    def _defineBuiltins (self, structures_module):
        assert self.isBuiltinNamespace()
        if not self.__definedBuiltins:
            from pyxb.namespace import builtin
            mr = self.lookupModuleRecordByUID(builtin.BuiltInObjectUID, create_if_missing=True, module_path=self.__builtinModulePath)
            self._defineBuiltins_ox(structures_module)
            self.__definedBuiltins = True
            mr.markIncorporated()
        return self

    def _loadComponentsFromArchives (self, structures_module):
        """Attempts to load the named objects held in this namespace.

        The base class implementation looks at the set of available archived
        namespaces, and if one contains this namespace unserializes its named
        object maps.

        Sub-classes may choose to look elsewhere, if this version fails or
        before attempting it.

        There is no guarantee that any particular category of named object has
        been located when this returns.  Caller must check.
        """
        for mr in self.moduleRecords():
            if mr.isLoadable():
                if mr.isPublic():
                    _log.info('Load %s from %s', mr, mr.archive())
                    try:
                        mr.archive().readNamespaces()
                    except pyxb.NamespaceArchiveError:
                        _log.exception("Failure reading namespaces in archive")
                else:
                    _log.info('Ignoring private module %s in validation', mr)
        self._activate()

    __didValidation = False
    __inValidation = False
    def validateComponentModel (self, structures_module=None):
        """Ensure this namespace is ready for use.

        If the namespace does not have a map of named objects, the system will
        attempt to load one.
        """
        if not self.__didValidation:
            # assert not self.__inValidation, 'Nested validation of %s' % (self.uri(),)
            if structures_module is None:
                import pyxb.xmlschema.structures as structures_module
            if self.isBuiltinNamespace():
                self._defineBuiltins(structures_module)
            try:
                self.__inValidation = True
                self._loadComponentsFromArchives(structures_module)
                self.__didValidation = True
            finally:
                self.__inValidation = False
        return True

    def _replaceComponent (self, existing_def, replacement_def):
        """Replace the existing definition with another.

        This is used in a situation where building the component model
        resulted in a new component instance being created and registered, but
        for which an existing component is to be preferred.  An example is
        when parsing the schema for XMLSchema itself: the built-in datatype
        components should be retained instead of the simple type definition
        components dynamically created from the schema.

        By providing the value C{None} as the replacement definition, this can
        also be used to remove components.

        @note: Invoking this requires scans of every item in every category
        map in the namespace.

        @return: C{replacement_def}
        """
        # We need to do replacements in the category map handler, the
        # resolver, and the component associator.
        return self._replaceComponent_csc(existing_def, replacement_def)

    def initialNamespaceContext (self):
        """Obtain the namespace context to be used when creating components in this namespace.

        Usually applies only to built-in namespaces, but is also used in the
        autotests when creating a namespace without a xs:schema element.  .
        Note that we must create the instance dynamically, since the
        information that goes into it has cross-dependencies that can't be
        resolved until this module has been completely loaded."""

        if self.__initialNamespaceContext is None:
            isn = { }
            if self.__contextInScopeNamespaces is not None:
                for (k, v) in six.iteritems(self.__contextInScopeNamespaces):
                    isn[k] = self.__identifyNamespace(v)
            kw = { 'target_namespace' : self
                 , 'default_namespace' : self.__identifyNamespace(self.__contextDefaultNamespace)
                 , 'in_scope_namespaces' : isn }
            self.__initialNamespaceContext = resolution.NamespaceContext(None, **kw)
        return self.__initialNamespaceContext


    def __identifyNamespace (self, nsval):
        """Identify the specified namespace, which should be a built-in.

        Normally we can just use a reference to the Namespace module instance,
        but when creating those instances we sometimes need to refer to ones
        for which the instance has not yet been created.  In that case, we use
        the name of the instance, and resolve the namespace when we need to
        create the initial context."""
        if nsval is None:
            return self
        if isinstance(nsval, six.string_types):
            nsval = globals().get(nsval)
        if isinstance(nsval, Namespace):
            return nsval
        raise pyxb.LogicError('Cannot identify namespace from %s' % (nsval,))

    def __str__ (self):
        if self.__uri is None:
            return 'AbsentNamespace%d' % (self.__absentNamespaceID,)
        assert self.__uri is not None
        if self.__boundPrefix is not None:
            rv = '%s=%s' % (self.__boundPrefix, self.__uri)
        else:
            rv = self.__uri
        return rv

from pyxb.namespace.builtin import XMLSchema_instance
from pyxb.namespace.builtin import XMLNamespaces
from pyxb.namespace.builtin import XMLSchema
from pyxb.namespace.builtin import XHTML
from pyxb.namespace.builtin import XML
from pyxb.namespace.builtin import XMLSchema_hfp
from pyxb.namespace.builtin import BuiltInObjectUID

resolution.NamespaceContext._AddTargetNamespaceAttribute(XMLSchema.createExpandedName('schema'), ExpandedName('targetNamespace'))

## Local Variables:
## fill-column:78
## End:



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Binding classes for types referenced by the attribute and attribute
group declarations in the XML namespace
(http://www.w3.org/XML/1998/namespace).

The content of this module was generated by PyXB and is coupled to the
components defined in L{pyxb.namespace.builtin._XML}.

"""

# ./pyxb/standard/bindings/raw/xml_.py
# PyXB bindings for NamespaceModule
# Generated 2009-07-25 13:37:53.257717 by PyXB version 0.5.2
import pyxb
import pyxb.binding
import pyxb.utils.utility
import pyxb.utils.domutils
from pyxb.utils import six

# Unique identifier for bindings created at the same time
_GenerationUID = pyxb.namespace.builtin.BuiltInObjectUID

# Import bindings for namespaces imported into schema

Namespace = pyxb.namespace.NamespaceForURI('http://www.w3.org/XML/1998/namespace', create_if_missing=True)

# Atomic SimpleTypeDefinition
class STD_ANON_space (pyxb.binding.datatypes.NCName, pyxb.binding.basis.enumeration_mixin):
    """No information"""

    _ExpandedName = None
STD_ANON_space._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=STD_ANON_space, enum_prefix=None)
STD_ANON_space.default = STD_ANON_space._CF_enumeration.addEnumeration(unicode_value=six.u('default'))
STD_ANON_space.preserve = STD_ANON_space._CF_enumeration.addEnumeration(unicode_value=six.u('preserve'))
STD_ANON_space._InitializeFacetMap(STD_ANON_space._CF_enumeration)

# Atomic SimpleTypeDefinition
class STD_ANON_emptyString (pyxb.binding.datatypes.string, pyxb.binding.basis.enumeration_mixin):
    """No information"""

    _ExpandedName = None
STD_ANON_emptyString._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=STD_ANON_emptyString, enum_prefix=None)
STD_ANON_emptyString.emptyString = STD_ANON_emptyString._CF_enumeration.addEnumeration(unicode_value=six.u(''))
STD_ANON_emptyString._InitializeFacetMap(STD_ANON_emptyString._CF_enumeration)

# Union SimpleTypeDefinition
# superclasses pyxb.binding.datatypes.anySimpleType
class STD_ANON_lang (pyxb.binding.basis.STD_union):
    """Simple type that is a union of pyxb.binding.datatypes.language, STD_ANON_emptyString"""

    _ExpandedName = None
    _MemberTypes = ( pyxb.binding.datatypes.language, STD_ANON_emptyString, )
STD_ANON_lang._CF_enumeration = pyxb.binding.facets.CF_enumeration(value_datatype=STD_ANON_lang)
STD_ANON_lang._CF_pattern = pyxb.binding.facets.CF_pattern()
STD_ANON_lang._InitializeFacetMap(STD_ANON_lang._CF_enumeration,
   STD_ANON_lang._CF_pattern)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
    <xs:element name="outer">
        <xs:complexType>
            <xs:sequence>
                <xs:element name="inner" type="xs:string"/>
            </xs:sequence>
        </xs:complexType>
    </xs:element>
</xs:schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0163 (unittest.TestCase):

    def testGood (self):
        instance = CreateFromDocument('<outer><inner>one</inner></outer>')
        self.assertEqual(instance.inner, 'one')

    def testBad (self):
        ran_test = True
        try:
            instance = CreateFromDocument('<outer><inner>one</inner><inner>extra</inner></outer>')
            ran_test = False
        except Exception as e:
            self.assertTrue(isinstance(e, pyxb.UnrecognizedContentError))
            self.assertTrue(isinstance(e.value, outer.typeDefinition()._UseForTag('inner').elementBinding().typeDefinition()))
            self.assertEqual(e.value, 'extra')
        self.assertTrue(ran_test)

if __name__ == '__main__':
    unittest.main()



__author__ = 'Harold Solbrig'

# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns="http://foo.org/test" targetNamespace="http://foo.org/test" elementFormDefault="qualified">
    <xs:complexType name="Outer">
        <xs:complexContent>
            <xs:extension base="Inner">
                <xs:sequence>
                    <xs:element name="c" type="xs:string" minOccurs="0" maxOccurs="1"/>
                </xs:sequence>
            </xs:extension>
        </xs:complexContent>
    </xs:complexType>
    <xs:element name="Test" type="Outer"/>
    <xs:complexType name="Inner">
        <xs:sequence>
            <xs:element name="a" type="xs:string" minOccurs="0"/>
            <xs:element name="b" type="xs:string" minOccurs="0"/>
        </xs:sequence>
    </xs:complexType>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

#pyxb.GlobalValidationConfig._setContentInfluencesGeneration(pyxb.GlobalValidationConfig.NEVER)

class TestTrac0184(unittest.TestCase):
   def testNesting(self):
       xml = """<?xml version="1.0" encoding="UTF-8"?>
<Test xmlns="http://foo.org/test">
<a>A</a>
<b>B</b>
<c>C</c>
</Test>"""
       instance = CreateFromDocument(xml)
       dom = instance.toDOM()
       xmlt = six.u('<?xml version="1.0" encoding="utf-8"?><ns1:Test xmlns:ns1="http://foo.org/test"><ns1:a>A</ns1:a><ns1:b>B</ns1:b><ns1:c>C</ns1:c></ns1:Test>')
       xmld = xmlt.encode('utf-8')
       self.assertEqual(instance.toxml('utf-8'), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:redefine/>
</xs:schema>'''

from pyxb.exceptions_ import *

import unittest

class TestTrac_0093 (unittest.TestCase):
    def testRedefine (self):
        self.assertRaises(pyxb.IncompleteImplementationError, pyxb.binding.generate.GeneratePython, schema_text=xsd)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns="urn:trac0112"
xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified"
targetNamespace="urn:trac0112" version="1.0">
     <xs:complexType name="BaseT" abstract="true"/>
     <xs:element name="Element" type="ElementType"/>
     <xs:complexType name="ElementType">
        <xs:complexContent>
            <xs:extension base="BaseT">
                <xs:sequence>
                    <xs:element name="Inner" minOccurs="0" maxOccurs="1">
                        <xs:complexType>
                            <xs:choice>
                                <xs:sequence>
                                    <xs:element maxOccurs="1" minOccurs="0" name="A1" type="xs:boolean"/>
                                    <xs:element maxOccurs="1" minOccurs="0" name="A2" type="xs:boolean"/>
                                </xs:sequence>
                                <xs:element maxOccurs="1" minOccurs="0" name="B" type="xs:boolean"/>
                                <xs:element maxOccurs="1" minOccurs="0" name="C" type="xs:boolean"/>
                            </xs:choice>
                        </xs:complexType>
                    </xs:element>
                </xs:sequence>
            </xs:extension>
        </xs:complexContent>
    </xs:complexType>
</xs:schema>
'''

xmlt = six.u('''<?xml version="1.0" encoding="utf-8"?><Element xmlns="urn:trac0112"><Inner><C>true</C></Inner></Element>''')
xmld = xmlt.encode('utf-8')


code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0112 (unittest.TestCase):
    def setUp (self):
        pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace(Namespace.uri())

    def tearDown (self):
        pyxb.utils.domutils.BindingDOMSupport.SetDefaultNamespace(None)

    def testExample (self):
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.toxml('utf-8'), xmld)

if __name__ == '__main__':
    unittest.main()



# Coding declaration for unicode strings
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
# See also:
# http://www.evanjones.ca/python-utf8.html
# http://bytes.com/topic/python/answers/41153-xml-unicode-what-am-i-doing-wrong

import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
import xml.sax
import io

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" elementFormDefault="qualified">
    <xs:element name="foo" type="xs:string"/>
    <xs:element name="bar">
        <xs:complexType>
            <xs:sequence>
                <xs:element name="e" type="xs:string" minOccurs="0"/>
            </xs:sequence>
            <xs:attribute name="a" type="xs:string"/>
        </xs:complexType>
    </xs:element>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0131 (unittest.TestCase):
    # Unicode string, UTF-8 encoding (per declaration at script top)
    strt = 'Sign of Leser-Trélat'
    strd = strt.encode('utf-8')
    base_xmlt = '<bar><e>' + strt + '</e></bar>'
    declared_xmlt = '<?xml version="1.0" encoding="UTF-8"?>' + base_xmlt

    def setUp (self):
        self.__xmlStyle = pyxb._XMLStyle

    def tearDown (self):
        pyxb._SetXMLStyle(self.__xmlStyle)

    def testRepresentation (self):
        self.assertEqual(self.strd, b'Sign of Leser-Tr\xc3\xa9lat')

    def testBasicParse (self):
        xmlt = self.base_xmlt
        xmld = xmlt.encode('utf-8')
        self.assertTrue(isinstance(xmlt, six.text_type))
        self.assertTrue(isinstance(xmld, six.binary_type))
        pyxb._SetXMLStyle(pyxb.XMLStyle_saxer)
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.e, self.strt)
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.e, self.strt)
        pyxb._SetXMLStyle(pyxb.XMLStyle_minidom)
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.e, self.strt)
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.e, self.strt)
        # saxdom can handle Unicode representation
        pyxb._SetXMLStyle(pyxb.XMLStyle_saxdom)
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.e, self.strt)
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.e, self.strt)

    def testDeclaredParse (self):
        xmlt = self.declared_xmlt
        xmld = xmlt.encode('utf-8')
        self.assertTrue(isinstance(xmlt, six.text_type))
        self.assertTrue(isinstance(xmld, six.binary_type))
        pyxb._SetXMLStyle(pyxb.XMLStyle_saxer)
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.e, self.strt)
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.e, self.strt)
        pyxb._SetXMLStyle(pyxb.XMLStyle_minidom)
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.e, self.strt)
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.e, self.strt)
        # saxdom can handle Unicode representation
        pyxb._SetXMLStyle(pyxb.XMLStyle_saxdom)
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.e, self.strt)
        instance = CreateFromDocument(xmld)
        self.assertEqual(instance.e, self.strt)

    def testElementEncode (self):
        instance = bar()
        instance.e = self.strt
        self.assertEqual(instance.e, self.strt)

    def testAttributeEncode (self):
        instance = bar()
        instance.a = self.strt
        self.assertEqual(instance.a, self.strt)

    def testuEncode (self):
        instance = foo(self.strt)
        self.assertEqual(instance, self.strt)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.utils.domutils import BindingDOMSupport
from pyxb.utils import six
import unittest

import bindings._A as A
import bindings._B as B

class Test (unittest.TestCase):
    def setUp (self):
        BindingDOMSupport.DeclareNamespace(A.Namespace, 'a')
        BindingDOMSupport.DeclareNamespace(B.Namespace, 'b')

    def tearDown (self):
        BindingDOMSupport.Reset()

    def tests (self):
        # Element {URN:nsdep:A}e1 uses Python identifier e1
        # Element {URN:nsdep:A}A_b_e1 uses Python identifier A_b_e1
        x = A.A_c_e1("A.b.e1", "e1")
        xmlt = six.u('<a:A_c_e1 xmlns:a="URN:nsdep:A"><a:A_b_e1>A.b.e1</a:A_b_e1><a:e1>e1</a:e1></a:A_c_e1>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(x.toxml("utf-8", root_only=True), xmld)
        self.assertEqual(A.tA_c, type(x))

        # Element {URN:nsdep:A}e1 uses Python identifier e1
        # Element {URN:nsdep:B}B_b_e1 uses Python identifier B_b_e1
        # Element {URN:nsdep:A}A_b_e1 uses Python identifier A_b_e1
        # Element {URN:nsdep:B}e1 uses Python identifier e1_
        y = B.B_c_e1("B.e1", "A.e1", "B.b.e1", x.A_b_e1)
        self.assertEqual('A.e1', y.e1)
        self.assertEqual('B.b.e1', y.B_b_e1)
        self.assertEqual('A.b.e1', y.A_b_e1)
        self.assertEqual(A.strA, type(y.A_b_e1))
        self.assertEqual('B.e1', y.e1_)

if '__main__' == __name__:
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema" targetNamespace="urn:tgt" xmlns:tgt="urn:tgt">
  <simpleType name="def">
    <restriction base="string">
      <length value="3"/>
    </restriction>
  </simpleType>
  <complexType name="class">
    <attribute  name="and" type="string"/>
    <attribute  name="value" type="string"/>
  </complexType>
  <element name="global" type="tgt:class"/>
</schema>'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

# Note: The generation phase should have printed:
#   Simple type {urn:tgt}def renamed to def_
#   Complex type {urn:tgt}class renamed to class_
#   Attribute {urn:tgt}class.and renamed to and_
#   Attribute {urn:tgt}class.value renamed to value_
#   Element {urn:tgt}global renamed to global_
class TestTrac_0087 (unittest.TestCase):
    def testReservedWords (self):
        # Element global
        self.assertTrue(isinstance(global_, pyxb.binding.basis.element))
        # Complex type class
        self.assertTrue(issubclass(class_, pyxb.binding.basis.complexTypeDefinition))
        # Simple type def
        self.assertTrue(issubclass(def_, pyxb.binding.datatypes.string))

    def testPyxbSymbols (self):
        xmls = '<tgt:global value="text" xmlns:tgt="urn:tgt"/>'
        instance = CreateFromDocument(xmls)
        self.assertTrue(isinstance(instance, class_))
        self.assertEqual(instance.value_, "text")

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils

import os.path

from pyxb.exceptions_ import *

import unittest

class TestIncludeDD (unittest.TestCase):
    def testDefault (self):
        schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-include-ad.xsd'))
        self.assertRaises(pyxb.SchemaValidationError, pyxb.binding.generate.GeneratePython, schema_location=schema_path)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import unittest

import trac169

import pyxb.utils.domutils

class TestTrac0169 (unittest.TestCase):

    def testBasic (self):
        v = trac169.c(23)
        self.assertEqual(23, v)

if __name__ == '__main__':
    unittest.main()



#  PYTHONPATH=../..:. PYXB_ARCHIVE_PATH=opengis/iso19139:+ ../../scripts/pyxbgen -u gmlapp.xsd -m gmlapp

from __future__ import print_function
import pyxb.bundles.opengis.gml_3_2 as gml
import gmlapp
import pyxb.utils.domutils

limits = gml.GridLimitsType(gml.GridEnvelopeType([0, 0], [4, 4]))
origin = gml.PointPropertyType(gml.Point(gml.pos([-93.25, 43.5]), id='_origin'))
offset_vector = gml.VectorType([0, 0])
grid = gml.RectifiedGrid(limits, 'latitude longitude', origin, offset_vector, dimension=2)
grid.id = '_%x' % id(grid)
domain = gml.domainSet(grid)

val_template = gmlapp.Temperature(nilReason='template', _nil=True, uom='urn:x-si:v1999:uom:degreesC')
cv = gml.CompositeValue(gml.valueComponents(AbstractValue=[val_template]))
cv.id = '_%s' % (id(cv),)
data = gml.tupleList('34.2 35.4')
range = gml.rangeSet(DataBlock=gml.DataBlockType(gml.rangeParameters(cv), data))

rgc = gml.RectifiedGridCoverage(domain, range)
rgc.id = '_%x' % (id(rgc),)

bds = pyxb.utils.domutils.BindingDOMSupport(namespace_prefix_map={ gml.Namespace : 'gml' , gmlapp.Namespace : 'app' })

xml = rgc.toxml("utf-8", bds=bds)
instance = gml.CreateFromDocument(xml)
bds.reset()
xml2 = instance.toxml("utf-8", bds=bds)

assert xml == xml2
print(xml2)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-include-aaq.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)
#open('code.py', 'w').write(code)
rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestIncludeDD (unittest.TestCase):
    def testDefault (self):
        xmls = '<entry><from>one</from><to>single</to></entry>'
        instance = CreateFromDocument(xmls)
        self.assertEqual(english.one, instance.from_)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import io
import time
import xml.dom.minidom
import pyxb.utils.saxdom
import pyxb.binding.saxer
import tmstvd
#import cProfile

# Extend the anonymous class used by the xtvd element to add a method
# we can use to test equality of two instances.  Normally, you'd just
# refer to the complex type binding class itself, but we don't know
# what PyXB named it.
class my_xtvd (tmstvd.xtvd.typeDefinition()):
    def equal (self, other, verbose=False):
        if len(self.stations.station) != len(other.stations.station):
            return False
        for i in range(len(self.stations.station)):
            s = self.stations.station[i]
            o = other.stations.station[i]
            if (s.callSign != o.callSign) or (s.name != o.name) or (s.id != o.id):
                return False
            if verbose:
                print('Match station %s is %s, id %d' % (s.callSign, s.name, s.id))
        return True
tmstvd.xtvd.typeDefinition()._SetSupersedingClass(my_xtvd)

# The sample document.
xml_file = 'tmsdatadirect_sample.xml'

print('Generating binding from %s with minidom' % (xml_file,))
mt1 = time.time()
xmld = open(xml_file, 'rb').read()
mt2 = time.time()
dom = xml.dom.minidom.parse(io.BytesIO(xmld))
mt3 = time.time()
#cProfile.run('tmstvd.CreateFromDOM(dom.documentElement)', 'dom.prf')
dom_instance = tmstvd.CreateFromDOM(dom.documentElement)
print('minidom first callSign at %s' %(dom_instance.stations.station[0].callSign._location(),))
mt4 = time.time()

print('Generating binding from %s with SAXDOM' % (xml_file,))
dt1 = time.time()
dom = pyxb.utils.saxdom.parse(io.BytesIO(xmld), location_base=xml_file)
dt2 = time.time()
#cProfile.run('tmstvd.CreateFromDOM(dom.documentElement)', 'saxdom.prf')
saxdom_instance = tmstvd.CreateFromDOM(dom.documentElement)
print('SAXDOM first callSign at %s' % (saxdom_instance.stations.station[0].callSign._location(),))
dt3 = time.time()

print('Generating binding from %s with SAX' % (xml_file,))
st1 = time.time()
saxer = pyxb.binding.saxer.make_parser(location_base=xml_file)
handler = saxer.getContentHandler()
st2 = time.time()
saxer.parse(io.BytesIO(xmld))
#cProfile.run('saxer.parse(open(xml_file))', 'sax.prf')
st3 = time.time()
sax_instance = handler.rootObject()
print('SAXER first callSign at %s' % (sax_instance.stations.station[0].callSign._location(),))

print('DOM-based read %f, parse %f, bind %f, total %f' % (mt2-mt1, mt3-mt2, mt4-mt3, mt4-mt2))
print('SAXDOM-based parse %f, bind %f, total %f' % (dt2-dt1, dt3-dt2, dt3-dt1))
print('SAX-based read %f, parse and bind %f, total %f' % (st2-st1, st3-st2, st3-st1))
print("Equality test on DOM vs SAX: %s" % (dom_instance.equal(sax_instance),))
print("Equality test on SAXDOM vs SAX: %s" % (saxdom_instance.equal(sax_instance, verbose=True),))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils

whatever = 'whatever200908021351'
import os.path
decl_xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema targetNamespace="%s"
  xmlns:whatever="whatever"
  xmlns:xs="http://www.w3.org/2001/XMLSchema">

 <xs:attribute name="lang">
  <xs:simpleType>
   <xs:union memberTypes="xs:language">
    <xs:simpleType>
     <xs:restriction base="xs:string">
      <xs:enumeration value=""/>
     </xs:restriction>
    </xs:simpleType>
   </xs:union>
  </xs:simpleType>
 </xs:attribute>
</xs:schema>
''' % (whatever,)

use_xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema" xmlns:whatever="%s">

 <xs:complexType name="ctd">
  <xs:attribute ref="whatever:lang"/>
 </xs:complexType>
</xs:schema>
''' % (whatever,)

import unittest

class TestBug_200908021351 (unittest.TestCase):
    def testBasic (self):
        generator = pyxb.binding.generate.Generator(allow_absent_module=True, generate_to_files=False)
        generator.addSchema(decl_xsd)
        generator.addModuleName('decl')
        generator.addSchema(use_xsd)
        generator.addModuleName('use')
        modules = generator.bindingModules()
        self.assertEqual(2, len(modules))
        ns = pyxb.namespace.NamespaceForURI(whatever)
        self.assertTrue(ns is not None)
        ad = ns.createExpandedName('lang').attributeDeclaration()
        self.assertTrue(ad is not None)
        self.assertEqual(2, len(ad.typeDefinition().memberTypeDefinitions()))

if __name__ == '__main__':
    unittest.main()



import trac193

bdv = trac193.Money._UseForTag('Value').elementBinding().typeDefinition()
rdv = trac193.OpenDeliveries._UseForTag('Value').elementBinding().typeDefinition()

assert issubclass(rdv, bdv)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
from pyxb.utils import unicode, xmlre
import re

import unittest

class TestXMLRE (unittest.TestCase):
    def assertMatches(self, xml_pattern, value):
        '''Helper function to assert a value matches an XSD regexp pattern.'''
        py_pattern = xmlre.XMLToPython(xml_pattern)
        compiled = re.compile(py_pattern)
        mo = compiled.match(value)
        self.assertTrue(mo is not None, 'XML re %r Python %r should match %r' % (xml_pattern, py_pattern, value))

    def assertNoMatch(self, xml_pattern, value):
        '''Helper function to assert a value does not matche an XSD regexp
        pattern.'''
        py_pattern = xmlre.XMLToPython(xml_pattern)
        compiled = re.compile(py_pattern)
        mo = compiled.match(value)
        self.assertTrue(mo is None, 'XML re %r Python %r should not match %r' % (xml_pattern, py_pattern, value))

    def testRangeErrors (self):
        self.assertTrue(xmlre.MaybeMatchCharacterClass('', 1) is None)

    def testWildcardEscape (self):
        (charset, position) = xmlre.MaybeMatchCharacterClass('.', 0)
        self.assertEqual(charset, unicode.WildcardEsc)
        self.assertEqual(position, 1)

    def testSingleCharEscapes (self):
        # 17 chars recognized as escapes
        self.assertEqual(len(unicode.SingleCharEsc), 17)

        (charset, position) = xmlre.MaybeMatchCharacterClass(r'\t', 0)
        self.assertEqual(charset.asTuples(), [ (9, 9) ])
        self.assertEqual(2, position)

        (charset, position) = xmlre.MaybeMatchCharacterClass(r'\?', 0)
        self.assertEqual(charset.asTuples(), [ (ord('?'), ord('?')) ])
        self.assertEqual(2, position)

        (charset, position) = xmlre.MaybeMatchCharacterClass(r'\\', 0)
        self.assertEqual(charset.asTuples(), [ (ord('\\'), ord('\\')) ])
        self.assertEqual(2, position)

    def testMultiCharEscapes (self):
        # 5*2 chars recognized as escapes
        self.assertEqual(len(unicode.MultiCharEsc), 10)
        (charset, position) = xmlre.MaybeMatchCharacterClass(r'\s', 0)
        self.assertEqual(charset.asTuples(), [ (9, 10), (13, 13), (32, 32) ])
        self.assertEqual(2, position)

    def testMatchCharProperty (self):
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassEsc, "\pL", 0)
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassEsc, "\p{L", 0)
        text = "\p{L}"
        (charset, position) = xmlre._MatchCharClassEsc(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, unicode.PropertyMap['L'])
        text = "\p{IsCyrillic}"
        (charset, position) = xmlre._MatchCharClassEsc(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, unicode.BlockMap['Cyrillic'])

    def testCharProperty (self):
        text = r'\p{D}'
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, text, 0)
        text = r'\P{D}'
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, text, 0)
        text = r'\p{N}'
        (charset, position) = xmlre.MaybeMatchCharacterClass(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, unicode.PropertyMap['N'])
        text = r'\P{N}'
        (charset, position) = xmlre.MaybeMatchCharacterClass(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset.negate(), unicode.PropertyMap['N'])
        text = r'\p{Sm}'
        (charset, position) = xmlre.MaybeMatchCharacterClass(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, unicode.PropertyMap['Sm'])

    def testCharBlock (self):
        text = r'\p{IsArrows}'
        (charset, position) = xmlre.MaybeMatchCharacterClass(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, unicode.BlockMap['Arrows'])
        text = r'\P{IsArrows}'
        (charset, position) = xmlre.MaybeMatchCharacterClass(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset.negate(), unicode.BlockMap['Arrows'])

        text = r'\p{IsWelsh}'
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, text, 0)
        text = r'\P{IsWelsh}'
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, text, 0)

    def testCharGroup (self):
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, '[]', 0)
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, '[A--]', 0)
        self.assertRaises(xmlre.RegularExpressionError, xmlre.MaybeMatchCharacterClass, '[A--]', 0)
        text = r'[A-Z]'
        #(charset, position) = xmlre.MaybeMatchCharacterClass(text, 0)
        #self.assertEqual(position, len(text))
        #self.assertEqual(charset, unicode.CodePointSet((ord('A'), ord('Z'))))

    def testCharOrSCE (self):
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassEsc, '[', 0)
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassEsc, ']', 0)
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassEsc, '-', 0)
        (charset, position) = xmlre._MatchCharClassEsc(r'\t', 0)
        self.assertEqual(2, position)
        self.assertEqual(unicode.CodePointSet("\t"), charset)

    def testMatchPosCharGroup (self):
        text = 'A]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 1)
        self.assertEqual(charset, unicode.CodePointSet(ord('A')))
        text = r'\n]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 2)
        self.assertEqual(charset, unicode.CodePointSet(10))
        text = r'-]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 1)
        self.assertEqual(charset, unicode.CodePointSet(ord('-')))
        text = 'A-Z]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 3)
        self.assertEqual(charset, unicode.CodePointSet((ord('A'), ord('Z'))))
        text = r'\t-\r]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 5)
        self.assertEqual(charset, unicode.CodePointSet((9, 13)))
        text = r'\t-A]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 4)
        self.assertEqual(charset, unicode.CodePointSet((9, ord('A'))))
        text = r'Z-\]]'
        (charset, has_sub, position) = xmlre._MatchPosCharGroup(text, 0)
        self.assertEqual(position, 4)
        self.assertEqual(charset, unicode.CodePointSet((ord('Z'), ord(']'))))
        text = 'Z-A]'
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchPosCharGroup, text, 0)

    def testMatchCharClassExpr (self):
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassExpr, 'missing open', 0)
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassExpr, '[missing close', 0)
        first_five = unicode.CodePointSet( (ord('A'), ord('E')) )
        text = r'[ABCDE]'
        (charset, position) = xmlre._MatchCharClassExpr(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, first_five)
        text = r'[^ABCDE]'
        (charset, position) = xmlre._MatchCharClassExpr(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset.negate(), first_five)
        text = r'[A-Z-[GHI]]'
        expected = unicode.CodePointSet( (ord('A'), ord('Z')) )
        expected.subtract( (ord('G'), ord('I') ))
        (charset, position) = xmlre._MatchCharClassExpr(text, 0)
        self.assertEqual(position, len(text))
        self.assertEqual(charset, expected)
        text = r'[\p{L}-\p{Lo}]'
        self.assertRaises(xmlre.RegularExpressionError, xmlre._MatchCharClassExpr, text, 0)
        text = r'[\p{L}-[\p{Lo}]]'
        (charset, position) = xmlre._MatchCharClassExpr(text, 0)
        expected = unicode.CodePointSet(unicode.PropertyMap['L'])
        expected.subtract(unicode.PropertyMap['Lo'])
        self.assertEqual(position, len(text))
        self.assertEqual(charset, expected)

    def testXMLToPython (self):
        self.assertEqual(r'^123$', xmlre.XMLToPython('123'))
        # Note that single-char escapes in the expression are
        # converted to character classes.
        self.assertEqual(r'^Why[ ]not[?]$', xmlre.XMLToPython(r'Why[ ]not\?'))

    def testRegularExpressions (self):
        text = '[\i-[:]][\c-[:]]*'
        compiled_re = re.compile(xmlre.XMLToPython(text))
        self.assertTrue(compiled_re.match('identifier'))
        self.assertFalse(compiled_re.match('0bad'))
        self.assertFalse(compiled_re.match(' spaceBad'))
        self.assertFalse(compiled_re.match('qname:bad'))
        text = '\\i\\c*'
        text_py = xmlre.XMLToPython(text)
        compiled_re = re.compile(text_py)
        self.assertTrue(compiled_re.match('identifier'))
        self.assertTrue(compiled_re.match('_underscore'))

    def testTrivialLiteral(self):
        # Simplest sanity check for assertMatches / assertNoMatch
        self.assertMatches("hello", "hello")
        self.assertNoMatch("hello", "hhello")
        self.assertNoMatch("hello", "helloo")
        self.assertNoMatch("hello", "goodbye")

    def testConvertingRangesToPythonWithDash(self):
        # It's really easy to convert this RE into "foo[&-X]bar", if
        # sorting characters in ASCII order without special-casing "-"
        self.assertNoMatch("foo[-&X]bar", "fooWbar")
        self.assertMatches("foo[-&X]bar", "foo-bar")
        self.assertMatches("foo[-&X]bar", "foo&bar")
        self.assertMatches("foo[-&X]bar", "fooXbar")

    def testConvertingRangesToPythonWithCaret(self):
        # It's really easy to convert this RE into "foo[^z]bar", if
        # sorting characters in ASCII order without special-casing "^"
        self.assertNoMatch("foo[z^]bar", "fooWbar")
        self.assertMatches("foo[z^]bar", "foozbar")
        self.assertMatches("foo[z^]bar", "foo^bar")

    def testConvertingRangesToPythonWithBackslash(self):
        # It's really easy to convert this RE into "foo[A\n]bar", if
        # you forget to special-case r"\"
        self.assertNoMatch("foo[A\\\\n]bar", "fooWbar")
        self.assertNoMatch("foo[A\\\\n]bar", "foo\nbar")
        self.assertMatches("foo[A\\\\n]bar", "fooAbar")
        self.assertMatches("foo[A\\\\n]bar", "foo\\bar")
        self.assertMatches("foo[A\\\\n]bar", "foonbar")

    def testCnUnicodeClass(self):
        # The Cn class is basically "everything that is not included in the
        # Unicode character database".  So it requires special handling when
        # you parse the Unicode character database.  It is really easy to
        # miss this and leave the Cn class empty.
        self.assertNoMatch("foo\\p{Cn}bar", "fooWbar")
        self.assertMatches("foo\\p{Cn}bar", "foo\ufffebar")
        self.assertMatches("foo\\P{Cn}bar", "fooWbar")
        self.assertNoMatch("foo\\P{Cn}bar", "foo\ufffebar")

    def testCnUnicodeClassInC(self):
        # If the Cn class is wrong (see above), then C will probably be wrong
        # too.
        self.assertNoMatch("foo\\p{C}bar", "fooWbar")
        self.assertMatches("foo\\p{C}bar", "foo\ufffebar")
        self.assertMatches("foo\\P{C}bar", "fooWbar")
        self.assertNoMatch("foo\\P{C}bar", "foo\ufffebar")

    def testMultiCharEscape_s(self):
        self.assertNoMatch("foo\\sbar", "fooWbar")
        self.assertMatches("foo\\sbar", "foo bar")

    def testMultiCharEscape_S(self):
        self.assertMatches("foo\\Sbar", "fooWbar")
        self.assertNoMatch("foo\\Sbar", "foo bar")

    def testMultiCharEscape_i(self):
        self.assertNoMatch("foo\\ibar", "foo bar")
        self.assertMatches("foo\\ibar", "fooWbar")
        self.assertMatches("foo\\ibar", "foo:bar")
        self.assertMatches("foo\\ibar", "foo_bar")
        self.assertMatches("foo\\ibar", "foo\u0D0Cbar")
        self.assertNoMatch("foo\\ibar", "foo-bar")
        self.assertNoMatch("foo\\ibar", "foo.bar")
        self.assertNoMatch("foo\\ibar", "foo\u203Fbar")
        self.assertNoMatch("foo\\ibar", "foo\u3005bar")

    def testMultiCharEscape_I(self):
        self.assertMatches("foo\\Ibar", "foo bar")
        self.assertNoMatch("foo\\Ibar", "fooWbar")
        self.assertNoMatch("foo\\Ibar", "foo:bar")
        self.assertNoMatch("foo\\Ibar", "foo_bar")
        self.assertNoMatch("foo\\Ibar", "foo\u0D0Cbar")
        self.assertMatches("foo\\Ibar", "foo-bar")
        self.assertMatches("foo\\Ibar", "foo.bar")
        self.assertMatches("foo\\Ibar", "foo\u203Fbar")
        self.assertMatches("foo\\Ibar", "foo\u3005bar")

    def testMultiCharEscape_c(self):
        self.assertNoMatch("foo\\cbar", "foo bar")
        self.assertMatches("foo\\cbar", "fooWbar")
        self.assertMatches("foo\\cbar", "foo:bar")
        self.assertMatches("foo\\cbar", "foo_bar")
        self.assertMatches("foo\\cbar", "foo\u0D0Cbar")
        self.assertMatches("foo\\cbar", "foo-bar")
        self.assertMatches("foo\\cbar", "foo.bar")
        self.assertNoMatch("foo\\cbar", "foo\u203Fbar")
        self.assertMatches("foo\\cbar", "foo\u3005bar")

    def testMultiCharEscape_C(self):
        self.assertMatches("foo\\Cbar", "foo bar")
        self.assertNoMatch("foo\\Cbar", "fooWbar")
        self.assertNoMatch("foo\\Cbar", "foo:bar")
        self.assertNoMatch("foo\\Cbar", "foo_bar")
        self.assertNoMatch("foo\\Cbar", "foo\u0D0Cbar")
        self.assertNoMatch("foo\\Cbar", "foo-bar")
        self.assertNoMatch("foo\\Cbar", "foo.bar")
        self.assertMatches("foo\\Cbar", "foo\u203Fbar")
        self.assertNoMatch("foo\\Cbar", "foo\u3005bar")

    def testMultiCharEscape_d(self):
        self.assertNoMatch("foo\\dbar", "foo bar")
        self.assertNoMatch("foo\\dbar", "foozbar")
        self.assertMatches("foo\\dbar", "foo5bar")
        self.assertMatches("foo\\dbar", "foo\u0669bar")

    def testMultiCharEscape_D(self):
        self.assertMatches("foo\\Dbar", "foo bar")
        self.assertMatches("foo\\Dbar", "foozbar")
        self.assertNoMatch("foo\\Dbar", "foo5bar")
        self.assertNoMatch("foo\\Dbar", "foo\u0669bar")

    def testMultiCharEscape_w(self):
        self.assertNoMatch("foo\\wbar", "foo bar")
        self.assertNoMatch("foo\\wbar", "foo&bar")
        self.assertMatches("foo\\wbar", "fooWbar")
        self.assertMatches("[\\w]*", "fooWboar")

    def testMultiCharEscape_W(self):
        self.assertMatches("foo\\Wbar", "foo bar")
        self.assertMatches("foo\\Wbar", "foo&bar")
        self.assertNoMatch("foo\\Wbar", "fooWbar")

    def testUnicodeClass(self):
        self.assertMatches("\\p{L}*", "hello")
        self.assertNoMatch("\\p{L}*", "hell7")

    def testQuotedOpenBrace(self):
        self.assertMatches("foo\\[bar", "foo[bar")
        self.assertNoMatch("foo\\[bar", "foo\\[bar")
        self.assertNoMatch("foo\\[bar", "foob")

    def testQuotedCloseBrace(self):
        self.assertMatches("foo\\]bar", "foo]bar")
        self.assertNoMatch("foo\\]bar", "foo\\]bar")
        self.assertNoMatch("foo\\]bar", "foob")

    def testQuotedAndUnquotedCloseBrace(self):
        self.assertMatches("foo[b\\]c]ar", "foobar")
        self.assertMatches("foo[b\\]c]ar", "foo]ar")
        self.assertMatches("foo[b\\]c]ar", "foocar")
        self.assertNoMatch("foo[b\\]c]ar", "fooar")

    def testUnquotedAndQuotedCloseBrace(self):
        self.assertMatches("foo[zb]c\\]ar", "foobc]ar")
        self.assertMatches("foo[zb]c\\]ar", "foozc]ar")
        self.assertNoMatch("foo[zb]c\\]ar", "foozar")

    def testQuotedOpenCloseBraces(self):
        self.assertMatches("foo\\[bar\\]", "foo[bar]")
        self.assertNoMatch("foo\\[bar\\]", "foo\\[bar]")
        self.assertNoMatch("foo\\[bar\\]", "foobar")

    def testQuotedAndUnquotedOpenBrace(self):
        self.assertMatches("foo\\[b[az]r", "foo[bar")
        self.assertMatches("foo\\[b[az]r", "foo[bzr")
        self.assertNoMatch("foo\\[b[az]r", "foobr")

    def testUnquotedAndQuotedOpenBrace(self):
        self.assertMatches("foo[b\\[az]r", "foobr")
        self.assertMatches("foo[b\\[az]r", "foo[r")
        self.assertNoMatch("foo[b\\[az]r", "foobar")

    def testFoo(self):
        self.assertMatches("foo\\\\[bc\\]a]r", "foo\\br")
        self.assertNoMatch("foo\\\\[bc\\]a]r", "foo\\bar")
        self.assertNoMatch("foo\\\\[bc\\]a]r", "foobar")

    def testDashStartRangeWithRange(self):
        # Spec says: The - character is a valid character range only at the
        # beginning or end of a positive character group.
        self.assertMatches("foo[-a-z]bar", "fooabar")
        self.assertMatches("foo[-a-z]bar", "foo-bar")
        self.assertMatches("foo[-a-z]bar", "foonbar")
        self.assertMatches("foo[-a-z]bar", "foozbar")
        self.assertNoMatch("foo[-a-z]bar", "fooWbar")

    def testDashStartRangeOneLetter(self):
        self.assertMatches("foo[-a]bar", "fooabar")
        self.assertMatches("foo[-a]bar", "foo-bar")
        self.assertNoMatch("foo[-a]bar", "fooWbar")

    def testDashStartRangeSeveralLetters(self):
        self.assertMatches("foo[-abc]bar", "fooabar")
        self.assertMatches("foo[-abc]bar", "foobbar")
        self.assertMatches("foo[-abc]bar", "foocbar")
        self.assertMatches("foo[-abc]bar", "foo-bar")
        self.assertNoMatch("foo[-abc]bar", "fooWbar")

    def testDashOnlyRange(self):
        self.assertMatches("foo[-]bar", "foo-bar")
        self.assertNoMatch("foo[-a-z]bar", "fooWbar")

    def testDashEndRange(self):
        self.assertMatches("foo[a-z-]bar", "fooabar")
        self.assertMatches("foo[a-z-]bar", "foo-bar")
        self.assertMatches("foo[a-z-]bar", "foonbar")
        self.assertMatches("foo[a-z-]bar", "foozbar")
        self.assertNoMatch("foo[a-z-]bar", "fooWbar")

    def testDashEndRangeOneLetter(self):
        self.assertMatches("foo[a-]bar", "fooabar")
        self.assertMatches("foo[a-]bar", "foo-bar")
        self.assertNoMatch("foo[a-]bar", "fooWbar")

    def testDashEndRangeSeveralLetters(self):
        self.assertMatches("foo[abc-]bar", "fooabar")
        self.assertMatches("foo[abc-]bar", "foobbar")
        self.assertMatches("foo[abc-]bar", "foocbar")
        self.assertMatches("foo[abc-]bar", "foo-bar")
        self.assertNoMatch("foo[abc-]bar", "fooWbar")

    def testDashEndRangeWithSub(self):
        self.assertMatches("foo[a-z--[q]]bar", "fooabar")
        self.assertMatches("foo[a-z--[q]]bar", "foo-bar")
        self.assertMatches("foo[a-z--[q]]bar", "foonbar")
        self.assertMatches("foo[a-z--[q]]bar", "foozbar")
        self.assertNoMatch("foo[a-z--[q]]bar", "fooWbar")
        self.assertNoMatch("foo[a-z--[q]]bar", "fooqbar")

    def testDashEndRangeOneLetterWithSub(self):
        self.assertMatches("foo[a--[q]]bar", "fooabar")
        self.assertMatches("foo[a--[q]]bar", "foo-bar")
        self.assertNoMatch("foo[a--[q]]bar", "fooWbar")

        self.assertMatches("foo[a--[a]]bar", "foo-bar")
        self.assertNoMatch("foo[a--[a]]bar", "fooabar")
        self.assertNoMatch("foo[a--[a]]bar", "fooWbar")

    def testDashEndRangeSeveralLettersWithSub(self):
        self.assertMatches("foo[abc--[b]]bar", "fooabar")
        self.assertMatches("foo[abc--[b]]bar", "foocbar")
        self.assertMatches("foo[abc--[b]]bar", "foo-bar")
        self.assertNoMatch("foo[abc--[b]]bar", "foobbar")
        self.assertNoMatch("foo[abc--[b]]bar", "fooWbar")

    def testCaret(self):
        self.assertMatches("foo^bar", "foo^bar")
        self.assertNoMatch("foo^bar", "foobar")
        self.assertNoMatch("foo^bar", "barfoo")

    def testCaretStart(self):
        self.assertMatches("^foobar", "^foobar")
        self.assertNoMatch("^foobar", "foobar")

    def testDollar(self):
        self.assertMatches("foo$bar", "foo$bar")
        self.assertNoMatch("foo$bar", "foobar")
        self.assertNoMatch("foo$bar", "barfoo")

    def testDollarEnd(self):
        self.assertMatches("foobar$", "foobar$")
        self.assertNoMatch("foobar$", "foobar")

    def testCaretInRangeSub(self):
        self.assertMatches("foo[a^-[a]]bar", "foo^bar")
        self.assertNoMatch("foo[a^-[a]]bar", "fooabar")
        self.assertNoMatch("foo[a^-[a]]bar", "foobar")

    def testCaretInRange(self):
        self.assertMatches("foo[a^]bar", "foo^bar")
        self.assertMatches("foo[a^]bar", "fooabar")
        self.assertNoMatch("foo[a^]bar", "foobar")

    def testSingleCharRange(self):
        self.assertMatches("foo[b]ar", "foobar")

    def testQuotedSingleChar(self):
        self.assertMatches("foo\\\\bar", "foo\\bar")

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<schema xmlns="http://www.w3.org/2001/XMLSchema">
  <element name="Element">
   <complexType>
     <attribute name="Required" type="string" use="required"/>
     <attribute name="Optional" type="string" use="optional"/>
   </complexType>
  </element>
</schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0126 (unittest.TestCase):
    def tearDown (self):
        pyxb.RequireValidWhenGenerating(True)
        pyxb.RequireValidWhenParsing(True)

    def testBasic (self):
        instance = Element()
        self.assertEqual(None, instance.Required)
        self.assertEqual(None, instance.Optional)

        pyxb.RequireValidWhenGenerating(False)
        xmlt = six.u('<Element/>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toDOM().documentElement.toxml("utf-8"), xmld)
        pyxb.RequireValidWhenGenerating(True)
        self.assertRaises(pyxb.MissingAttributeError, instance.toDOM)
        instance.Required = 'value'
        xmlt = six.u('<Element Required="value"/>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(instance.toDOM().documentElement.toxml("utf-8"), xmld)


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node
import pyxb.binding.datatypes as xs

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="wrapper">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="holding">
          <xs:complexType>
            <xs:sequence>
              <xs:element name="optional" minOccurs="0">
                <xs:complexType>
                  <xs:simpleContent>
                    <xs:extension base="xs:int">
                      <xs:attribute name="deep" type="xs:int"/>
                    </xs:extension>
                  </xs:simpleContent>
                </xs:complexType>
              </xs:element>
              <xs:element name="required">
                <xs:complexType>
                  <xs:simpleContent>
                    <xs:extension base="xs:string">
                      <xs:attribute name="deep" type="xs:int"/>
                    </xs:extension>
                  </xs:simpleContent>
                </xs:complexType>
              </xs:element>
            </xs:sequence>
            <xs:attribute name="inner" type="xs:int"/>
          </xs:complexType>
        </xs:element>
      </xs:sequence>
      <xs:attribute name="outer" type="xs:int"/>
    </xs:complexType>
  </xs:element>
  <xs:element name="shallow">
    <xs:complexType>
      <xs:sequence>
        <xs:element name="optional" minOccurs="0">
          <xs:complexType>
            <xs:simpleContent>
              <xs:extension base="xs:int">
                <xs:attribute name="deep" type="xs:int"/>
              </xs:extension>
            </xs:simpleContent>
          </xs:complexType>
        </xs:element>
      </xs:sequence>
      <xs:attribute name="outer" type="xs:int"/>
    </xs:complexType>
  </xs:element>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *
from pyxb import BIND

import unittest

def SET_optional (instance, value):
    instance.optional = value

class TestTrac0039 (unittest.TestCase):
    """Creating nested anonymous elements"""
    def testShallowSet (self):
        w = shallow()
        w.optional = 4
        self.assertEqual(w.optional.value(), 4)
        self.assertTrue(w.optional.deep is None)
        w.optional = BIND(5)
        self.assertEqual(w.optional.value(), 5)
        self.assertTrue(w.optional.deep is None)
        self.assertTrue(isinstance(w.optional.value(), xs.int))
        self.assertRaises(pyxb.SimpleTypeValueError, SET_optional, w, BIND('string'))
        w.optional = BIND(6, deep=1)
        self.assertEqual(w.optional.value(), 6)
        self.assertEqual(w.optional.deep, 1)

    def testShallowCTOR (self):
        w = shallow(BIND(5))
        self.assertTrue(isinstance(w.optional.value(), xs.int))
        self.assertEqual(w.optional.value(), 5)
        w = shallow(6)
        self.assertTrue(isinstance(w.optional.value(), xs.int))
        self.assertEqual(w.optional.value(), 6)
        self.assertRaises(pyxb.UnrecognizedContentError, shallow, BIND('string'))

    def testDeep (self):
        w = wrapper(BIND(BIND(4, deep=4), BIND('hi')))
        xmlt = six.u('<wrapper><holding><optional deep="4">4</optional><required>hi</required></holding></wrapper>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(w.toxml("utf-8", root_only=True), xmld)
        w = wrapper(BIND(BIND('hi', deep=2)))
        xmlt = six.u('<wrapper><holding><required deep="2">hi</required></holding></wrapper>')
        xmld = xmlt.encode('utf-8')
        self.assertEqual(w.toxml("utf-8", root_only=True), xmld)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from xml.dom import Node

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="a"/>
  <xs:element name="b"/>
  <xs:element name="c"/>
  <xs:element name="d"/>
  <xs:element name="e"/>
  <xs:group name="Cabc">
    <xs:choice>
      <xs:element ref="a"/>
      <xs:element ref="b"/>
      <xs:element ref="c"/>
    </xs:choice>
  </xs:group>
  <xs:group name="Cbcd">
    <xs:choice>
      <xs:element ref="b"/>
      <xs:element ref="c"/>
      <xs:element ref="d"/>
    </xs:choice>
  </xs:group>
  <xs:group name="Cbe">
    <xs:choice>
      <xs:element ref="b"/>
      <xs:element ref="e"/>
    </xs:choice>
  </xs:group>
  <xs:group name="CabcPCbcdPCbe">
    <xs:sequence>
      <xs:group ref="Cabc"/>
      <xs:group ref="Cbcd"/>
      <xs:group ref="Cbe"/>
    </xs:sequence>
  </xs:group>
  <xs:group name="CbcdPCbe">
    <xs:sequence>
      <xs:group ref="Cbcd"/>
      <xs:group ref="Cbe"/>
    </xs:sequence>
  </xs:group>
  <xs:complexType name="aBCde">
    <xs:sequence>
      <xs:group ref="CabcPCbcdPCbe"/>
    </xs:sequence>
  </xs:complexType>
  <xs:complexType name="Bcde">
    <xs:sequence>
      <xs:group ref="CbcdPCbe"/>
    </xs:sequence>
  </xs:complexType>
  <xs:complexType name="aBCDE">
    <xs:sequence>
      <xs:group ref="CabcPCbcdPCbe"/>
      <xs:group ref="CbcdPCbe"/>
    </xs:sequence>
  </xs:complexType>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0034 (unittest.TestCase):
    def test_aBCde (self):
        instance = aBCde()
        self.assertEqual(None, instance.a)
        self.assertEqual([], instance.b)
        self.assertEqual([], instance.c)
        self.assertEqual(None, instance.d)
        self.assertEqual(None, instance.e)

    def test_Bcde (self):
        instance = Bcde()
        self.assertEqual([], instance.b)
        self.assertEqual(None, instance.c)
        self.assertEqual(None, instance.d)
        self.assertEqual(None, instance.e)

    def test_aBCDE (self):
        instance = aBCDE()
        self.assertEqual(None, instance.a)
        self.assertEqual([], instance.b)
        self.assertEqual([], instance.c)
        self.assertEqual([], instance.d)
        self.assertEqual([], instance.e)

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils

from xml.dom import Node

import os.path
schema_path = os.path.abspath(os.path.join(os.path.dirname(__file__), '../schemas/test-wildcard.xsd'))
code = pyxb.binding.generate.GeneratePython(schema_location=schema_path)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest


def nc_not (ns_or_absent):
    return ( pyxb.xmlschema.structures.Wildcard.NC_not, ns_or_absent )

class TestIntensionalSet (unittest.TestCase):

    def testTest (self):
        ns = 'URN:namespace'
        not_nc = nc_not(ns)
        self.assertTrue(isinstance(not_nc, tuple))
        self.assertEqual(2, len(not_nc))
        self.assertEqual(pyxb.xmlschema.structures.Wildcard.NC_not, not_nc[0])
        self.assertEqual(ns, not_nc[1])

    def testUnion_1 (self):
        UNION = pyxb.xmlschema.structures.Wildcard.IntensionalUnion
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_any, UNION([ nc_any, nc_any ]))
        self.assertEqual(nc_not(ns1), UNION([ nc_not(ns1), nc_not(ns1) ]))
        self.assertEqual(set([ns1]), UNION([ set([ns1]), set([ns1]) ]))

    def testUnion_2 (self):
        UNION = pyxb.xmlschema.structures.Wildcard.IntensionalUnion
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_any, UNION([ nc_any, set([ns1]) ]))
        self.assertEqual(nc_any, UNION([ nc_any, nc_not(ns1) ]))
        self.assertEqual(nc_any, UNION([ nc_any, nc_not(None) ]))

    def testUnion_3 (self):
        UNION = pyxb.xmlschema.structures.Wildcard.IntensionalUnion
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(set([ns1, ns2]), UNION([set([ns1]), set([ns2])]))
        self.assertEqual(set([None, ns1]), UNION([set([None]), set([ns1])]))
        self.assertEqual(set([None]), UNION([set([None]), set([None])]))

    def testUnion_4 (self):
        UNION = pyxb.xmlschema.structures.Wildcard.IntensionalUnion
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_not(None), UNION([nc_not(ns1), nc_not(ns2)]))
        self.assertEqual(nc_not(None), UNION([nc_not(ns1), nc_not(None)]))

    def testUnion_5 (self):
        UNION = pyxb.xmlschema.structures.Wildcard.IntensionalUnion
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_any, UNION([nc_not(ns1), set([ns1, None])])) # 5.1
        self.assertEqual(nc_not(None), UNION([nc_not(ns1), set([ns1, ns2])])) # 5.2
        self.assertRaises(SchemaValidationError, UNION, [nc_not(ns1), set([None, ns2])]) # 5.3
        self.assertEqual(nc_not(ns1), UNION([nc_not(ns1), set([ns2])])) # 5.4

    def testUnion_6 (self):
        UNION = pyxb.xmlschema.structures.Wildcard.IntensionalUnion
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_any, UNION([nc_not(None), set([ns1, ns2, None])])) # 6.1
        self.assertEqual(nc_not(None), UNION([nc_not(None), set([ns1, ns2])])) # 6.2

    def testIntersection_1 (self):
        ISECT = pyxb.xmlschema.structures.Wildcard.IntensionalIntersection
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_any, ISECT([ nc_any, nc_any ]))
        self.assertEqual(nc_not(ns1), ISECT([ nc_not(ns1), nc_not(ns1) ]))
        self.assertEqual(set([ns1]), ISECT([ set([ns1]), set([ns1]) ]))

    def testIntersection_2 (self):
        ISECT = pyxb.xmlschema.structures.Wildcard.IntensionalIntersection
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(set([ns1]), ISECT([ nc_any, set([ns1]) ]))
        self.assertEqual(nc_not(ns1), ISECT([ nc_any, nc_not(ns1) ]))
        self.assertEqual(nc_not(None), ISECT([ nc_any, nc_not(None) ]))

    def testIntersection_3 (self):
        ISECT = pyxb.xmlschema.structures.Wildcard.IntensionalIntersection
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(set([ns2]), ISECT([nc_not(ns1), set([ns1, ns2, None])]))
        self.assertEqual(set([ns2]), ISECT([nc_not(ns1), set([ns1, ns2])]))
        self.assertEqual(set([ns2]), ISECT([nc_not(ns1), set([ns2])]))

    def testIntersection_4 (self):
        ISECT = pyxb.xmlschema.structures.Wildcard.IntensionalIntersection
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(set([ns2]), ISECT([set([ns1, ns2]), set([ns2, None])]))
        self.assertEqual(set([ns2, None]), ISECT([set([None, ns1, ns2]), set([ns2, None])]))
        self.assertEqual(set([]), ISECT([set([ns1]), set([ns2, None])]))
        self.assertEqual(set([]), ISECT([set([ns1]), set([ns2, ns1]), set([ns2, None])]))
        self.assertEqual(set([ns1]), ISECT([set([ns1, None]), set([None, ns2, ns1]), set([ns1, ns2])]))

    def testIntersection_5 (self):
        ISECT = pyxb.xmlschema.structures.Wildcard.IntensionalIntersection
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertRaises(SchemaValidationError, ISECT, [nc_not(ns1), nc_not(ns2)])

    def testIntersection_6 (self):
        ISECT = pyxb.xmlschema.structures.Wildcard.IntensionalIntersection
        nc_any = pyxb.xmlschema.structures.Wildcard.NC_any
        ns1 = 'URN:first'
        ns2 = 'URN:second'
        self.assertEqual(nc_not(ns1), ISECT([nc_not(ns1), nc_not(None)]))

class TestWildcard (unittest.TestCase):
    def setUp (self):
        # Hide the warning about failure to convert DOM node {}third
        # to a binding
        self.__basis_log = logging.getLogger('pyxb.binding.basis')
        self.__basis_loglevel = self.__basis_log.level
        self.__basis_log.setLevel(logging.ERROR)

    def tearDown (self):
        pyxb.RequireValidWhenParsing(True)
        self.__basis_log.level = self.__basis_loglevel

    def testElement (self):
        # NB: Test on CTD, not element
        self.assertTrue(wrapper_._HasWildcardElement)
        xmls = '<wrapper><first/><second/><third/></wrapper>'
        doc = pyxb.utils.domutils.StringToDOM(xmls)
        instance = wrapper.createFromDOM(doc.documentElement)
        self.assertTrue(isinstance(instance.wildcardElements(), list))
        self.assertEqual(1, len(instance.wildcardElements()))
        # Alternative parser path
        instance = CreateFromDocument(xmls)
        self.assertTrue(isinstance(instance.wildcardElements(), list))
        self.assertEqual(1, len(instance.wildcardElements()))

    def _validateWildcardWrappingRecognized (self, instance):
        self.assertTrue(isinstance(instance.wildcardElements(), list))
        self.assertEqual(1, len(instance.wildcardElements()))
        dom = instance.wildcardElements()[0]
        self.assertTrue(isinstance(dom, Node))
        self.assertEqual(Node.ELEMENT_NODE, dom.nodeType)
        self.assertEqual('third', dom.nodeName)
        self.assertEqual(1, len(dom.childNodes))
        cdom = dom.firstChild
        self.assertTrue(isinstance(cdom, Node))
        self.assertEqual(Node.ELEMENT_NODE, cdom.nodeType)
        self.assertEqual('selt', cdom.nodeName)
        ccdom = cdom.firstChild
        self.assertTrue(isinstance(ccdom, Node))
        self.assertEqual(Node.TEXT_NODE, ccdom.nodeType)
        self.assertEqual('text', ccdom.data)

    def testWildcardWrappingRecognized (self):
        # NB: Test on CTD, not element
        self.assertTrue(wrapper_._HasWildcardElement)
        xmls = '<wrapper><first/><second/><third><selt>text</selt></third></wrapper>'
        doc = pyxb.utils.domutils.StringToDOM(xmls)
        instance = wrapper.createFromDOM(doc.documentElement)
        self._validateWildcardWrappingRecognized(instance)
        # Alternative parser path
        instance = CreateFromDocument(xmls)
        self._validateWildcardWrappingRecognized(instance)

    def testMultiElement (self):
        tested_overmax = False
        for rep in range(0, 6):
            xmls = '<wrapper><first/><second/>%s</wrapper>' % (''.join(rep * ['<third/>']),)
            doc = pyxb.utils.domutils.StringToDOM(xmls)
            if 3 >= rep:
                instance = wrapper.createFromDOM(doc.documentElement)
                self.assertTrue(isinstance(instance.wildcardElements(), list))
                self.assertEqual(rep, len(instance.wildcardElements()))
                for i in range(0, rep):
                    self.assertEqual('third', instance.wildcardElements()[i].nodeName)
            else:
                tested_overmax = True
                self.assertRaises(UnrecognizedContentError, wrapper.createFromDOM, doc.documentElement)
        self.assertTrue(tested_overmax)

    def testAttribute (self):
        # NB: Test on CTD, not element
        self.assertTrue(isinstance(wrapper_._AttributeWildcard, pyxb.binding.content.Wildcard))
        xmls = '<wrapper myattr="true" auxattr="somevalue"/>'
        doc = pyxb.utils.domutils.StringToDOM(xmls)
        instance = wrapper.createFromDOM(doc.documentElement)
        self.assertTrue(isinstance(instance.wildcardAttributeMap(), dict))
        self.assertEqual(1, len(instance.wildcardAttributeMap()))
        self.assertEqual('somevalue', instance.wildcardAttributeMap()['auxattr'])

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
# Copyright 2009-2013, Peter A. Bigot
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain a
# copy of the License at:
#
#            http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""Utility functions related to U{XML Namespaces<http://www.w3.org/TR/2006/REC-xml-names-20060816/index.html>}."""

import logging
import pyxb
import pyxb.namespace
from pyxb.utils import six

_log = logging.getLogger(__name__)

def NamespaceInstance (namespace):
    """Get a namespace instance for the given namespace.

    This is used when it is unclear whether the namespace is specified by URI
    or by instance or by any other mechanism we might dream up in the
    future."""
    if isinstance(namespace, pyxb.namespace.Namespace):
        return namespace
    if isinstance(namespace, six.string_types):
        return NamespaceForURI(namespace, True)
    raise pyxb.LogicError('Cannot identify namespace from value of type %s' % (type(namespace),))

def NamespaceForURI (uri, create_if_missing=False):
    """Given a URI, provide the L{Namespace} instance corresponding to it.

    This can only be used to lookup or create real namespaces.  To create
    absent namespaces, use L{CreateAbsentNamespace}.

    @param uri: The URI that identifies the namespace
    @type uri: A non-empty C{str} or C{unicode} string
    @keyword create_if_missing: If C{True}, a namespace for the given URI is
    created if one has not already been registered.  Default is C{False}.
    @type create_if_missing: C{bool}
    @return: The Namespace corresponding to C{uri}, if available
    @rtype: L{Namespace} or C{None}
    @raise pyxb.LogicError: The uri is not a non-empty string
    """
    if not isinstance(uri, six.string_types):
        raise pyxb.LogicError('Cannot lookup absent namespaces')
    if 0 == len(uri):
        raise pyxb.LogicError('Namespace URIs must not be empty strings')
    rv = pyxb.namespace.Namespace._NamespaceForURI(uri)
    if (rv is None) and create_if_missing:
        rv = pyxb.namespace.Namespace(uri)
    return rv

def CreateAbsentNamespace ():
    """Create an absent namespace.

    Use this when you need a namespace for declarations in a schema with no
    target namespace.  Absent namespaces are not stored in the infrastructure;
    it is your responsibility to hold on to the reference you get from this,
    because you won't be able to look it up."""
    return pyxb.namespace.Namespace.CreateAbsentNamespace()

def AvailableNamespaces ():
    """Return the complete set of Namespace instances known to the system."""
    return pyxb.namespace.Namespace.AvailableNamespaces()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import ucum
import pyxb
import sys
from pyxb.utils import six

def ucumHTML (u):
    """Convert mixed content to UCUM's version of HTML"""
    txt = []
    if isinstance(u, six.text_type):
        return u.strip()
    for c in u.orderedContent():
        if isinstance(c, pyxb.binding.basis.NonElementContent):
            txt.append(c.value.strip())
        else:
            en = c.value._element().name().localName()
            txt.append('<{}>{}</{}>'.format(en, ucumHTML(c.value), en))
    return ''.join(txt)

try:
    instance = ucum.CreateFromDocument(open('ucum-essence.xml').read())
except pyxb.ValidationError as e:
    print(e.details())
    sys.exit(1)

units = []
units.extend(instance.base_unit)
units.extend(instance.unit)
for u in units:
    if isinstance(u, ucum.base_unit.typeDefinition()):
        print(u.Code)
    elif 1 == u.value_.value_:
        print('{}\talias {}'.format(u.Code, u.value_.Unit))
    elif u.value_.function is not None:
        print('{}\tas value {}'.format(u.Code, u.value_.Unit))
    else:
        print('{}\tas {} times {}'.format(u.Code, u.value_.value_, u.value_.Unit))
    ps = ''
    if u.printSymbol is not None:
        ps = ucumHTML(u.printSymbol)
    if ps:
        print('\tprints as: {}'.format(ps))



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
import operator
import functools
from xml.dom import Node
from pyxb.utils.six.moves import xrange

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
 <xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:element name="eST" type="xs:int"/>
  <xs:element name="eCT">
    <xs:complexType>
      <xs:simpleContent>
        <xs:extension base="xs:int">
          <xs:attribute name="units" type="xs:string" use="optional"/>
        </xs:extension>
      </xs:simpleContent>
    </xs:complexType>
  </xs:element>
  <xs:element name="eSTs">
    <xs:complexType>
      <xs:sequence>
        <xs:element ref="eST" minOccurs="0" maxOccurs="unbounded"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
  <xs:element name="eCTs">
    <xs:complexType>
      <xs:sequence>
        <xs:element ref="eCT" minOccurs="0" maxOccurs="unbounded"/>
      </xs:sequence>
    </xs:complexType>
  </xs:element>
</xs:schema>'''

code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0175 (unittest.TestCase):
    def testBasic (self):
        s = eST(1)
        self.assertEqual(s, 1)
        c = eCT(2)
        self.assertEqual(c.value(), 2)
        instance = eSTs(1,2,3,4)
        self.assertEqual(4, len(instance.eST))
        self.assertTrue(functools.reduce(operator.iand, map(lambda _i: operator.eq(1+_i, instance.eST[_i]), xrange(len(instance.eST))), True))
        instance = eCTs(1,2,3,4)
        self.assertEqual(4, len(instance.eCT))
        self.assertTrue(functools.reduce(operator.iand, map(lambda _i: operator.eq(1+_i, instance.eCT[_i].value()), xrange(len(instance.eCT))), True))

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
import pyxb.binding.generate
import pyxb.utils.domutils
from pyxb.utils import six
from xml.dom import Node

import os.path
xst = '''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
  <xs:complexType name="tInt">
    <xs:simpleContent>
      <xs:extension base="xs:int">
        <xs:attribute name="units" type="xs:string"/>
      </xs:extension>
    </xs:simpleContent>
  </xs:complexType>
  <xs:element name="Int" type="tInt"/>
</xs:schema>
'''

code = pyxb.binding.generate.GeneratePython(schema_text=xst)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac0213 (unittest.TestCase):
    def testMissingContent (self):
        import copy

        xmlt = six.u('<Int units="m">32</Int>')
        xmld = xmlt.encode('utf-8')
        instance = CreateFromDocument(xmlt)
        self.assertEqual(instance.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(instance.value(), 32)
        self.assertEqual(instance.units, "m")

        ni = copy.copy(instance)
        self.assertEqual(ni.toxml('utf-8', root_only=True), xmld)
        self.assertEqual(ni.value(), 32)
        self.assertEqual(ni.units, "m")
        ni.reset()
        if sys.version_info[:2] < (2, 7):
            self.assertRaises(SimpleContentAbsentError, ni.validateBinding)
        else:
            with self.assertRaises(SimpleContentAbsentError) as cm:
                ni.validateBinding()
            e = cm.exception
            self.assertEqual(e.instance, ni)

if __name__ == '__main__':
    unittest.main()



# -*- coding: iso-2022-jp -*-
from __future__ import unicode_literals
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
#

import sys
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.saxutils
from pyxb.utils import six
import tempfile
import xml.sax

import os.path
xsd='''<?xml version="1.0" encoding="utf-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
        <xs:element name="text" type="xs:string"/>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code.encode('utf-8'), 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_0139 (unittest.TestCase):
    ascii_enc = sys.getdefaultencoding()
    asciit = 'something'
    nihongo_enc = 'iso-2022-jp'
    nihongot = '基盤地図情報ダウンロードデータ（GML版）'

    def buildDocument (self, text, encoding):
        map = { 'text' : text }
        if encoding is None:
            map['encoding'] = ''
        else:
            map['encoding'] = ' encoding="%s"' % (encoding,)
        return '<?xml version="1.0"%(encoding)s?><text>%(text)s</text>' % map

    # NOTE: Init-lower version does not exist before Python 2.7, so
    # make this non-standard and invoke it in init
    def SetUpClass (self):
        self.nihongo_xmlt = self.buildDocument(self.nihongot, self.nihongo_enc)
        (fd, self.path_nihongo) = tempfile.mkstemp()
        self.nihongo_xmld = self.nihongo_xmlt.encode(self.nihongo_enc)
        os.fdopen(fd, 'wb').write(self.nihongo_xmld)
        self.ascii_xmlt = self.buildDocument(self.asciit, self.ascii_enc)
        (fd, self.path_ascii) = tempfile.mkstemp()
        self.ascii_xmld = self.ascii_xmlt.encode(self.ascii_enc)
        os.fdopen(fd, 'wb').write(self.ascii_xmld)

        # Ensure test failures are not due to absence of libxml2,
        # which PyXB can't control.
        self.have_libxml2 = True
        try:
            import drv_libxml2
        except ImportError:
            self.have_libxml2 = False

    # NOTE: Init-lower version does not exist before Python 2.7, so
    # make this non-standard and invoke it in del
    def TearDownClass (self):
        os.remove(self.path_ascii)
        os.remove(self.path_nihongo)

    def useLibXML2Parser (self):
        pyxb.utils.saxutils.SetCreateParserModules(['drv_libxml2'])

    def tearDown (self):
        pyxb.utils.saxutils.SetCreateParserModules(None)

    def __init__ (self, *args, **kw):
        self.SetUpClass()
        super(TestTrac_0139, self).__init__(*args, **kw)

    def __del__ (self, *args, **kw):
        self.TearDownClass()
        try:
            super(TestTrac_0139, self).__del__(*args, **kw)
        except AttributeError:
            pass

    # Make sure create parser modules is reset after each test
    def tearDown (self):
        pyxb.utils.saxutils.SetCreateParserModules(None)

    def testParserTypes (self):
        import sys
        if sys.version_info < (3, 0):
            default_enc = 'ascii'
        else:
            default_enc = 'utf-8'
        self.assertEqual(default_enc, sys.getdefaultencoding())
        parser = pyxb.utils.saxutils.make_parser()
        self.assertTrue(isinstance(parser, xml.sax.expatreader.ExpatParser))
        if self.have_libxml2:
            import drv_libxml2
            self.useLibXML2Parser()
            parser = pyxb.utils.saxutils.make_parser()
            self.assertTrue(isinstance(parser, drv_libxml2.LibXml2Reader))

    def testASCII_expat_text (self):
        instance = CreateFromDocument(self.ascii_xmlt)
        self.assertEqual(self.asciit, instance)

    def testASCII_expat_data (self):
        instance = CreateFromDocument(self.ascii_xmld)
        self.assertEqual(self.asciit, instance)

    def testASCII_libxml2_str (self):
        if not self.have_libxml2:
            _log.warning('%s: testASCII_libxml2_str bypassed since libxml2 not present', __file__)
            return
        self.useLibXML2Parser()
        instance = CreateFromDocument(self.ascii_xmld)
        self.assertEqual(self.asciit, instance)

    def testASCII_expat_file (self):
        xmld = open(self.path_ascii, 'rb').read()
        instance = CreateFromDocument(xmld)
        self.assertEqual(self.asciit, instance)

    def testASCII_libxml2_file (self):
        if not self.have_libxml2:
            _log.warning('%s: testASCII_libxml2_file bypassed since libxml2 not present', __file__)
            return
        self.useLibXML2Parser()
        xmld = open(self.path_ascii, 'rb').read()
        instance = CreateFromDocument(xmld)
        self.assertEqual(self.asciit, instance)

    def testNihongo_expat_text (self):
        self.assertRaises(xml.sax.SAXParseException, CreateFromDocument, self.nihongo_xmlt)

    def testNihongo_expat_data (self):
        self.assertRaises(xml.sax.SAXParseException, CreateFromDocument, self.nihongo_xmld)

    def testNihongo_expat_file (self):
        xmld = open(self.path_nihongo, 'rb').read()
        self.assertRaises(xml.sax.SAXParseException, CreateFromDocument, xmld)

    def testNihongo_libxml2_str (self):
        if not self.have_libxml2:
            _log.warning('%s: testNihongo_libxml2_str bypassed since libxml2 not present', __file__)
            return
        self.assertRaises(xml.sax.SAXParseException, CreateFromDocument, self.nihongo_xmlt)
        # ERROR: This should be fine, see trac/147
        #instance = CreateFromDocument(self.nihongo_xmld)
        #self.assertEqual(self.nihongot, instance)
        self.assertRaises(xml.sax.SAXParseException, CreateFromDocument, self.nihongo_xmld)

    def testNihongo_libxml2_file (self):
        if not self.have_libxml2:
            _log.warning('%s: testNihongo_libxml2_file bypassed since libxml2 not present', __file__)
            return
        self.useLibXML2Parser()
        xmld = open(self.path_nihongo, 'rb').read()
        instance = CreateFromDocument(xmld)
        self.assertEqual(self.nihongot, instance)

    def testASCII_textio (self):
        f = open(self.path_ascii).read()
        sio = io.StringIO(self.ascii_xmlt).read()
        self.assertEqual(f, sio)

    def testASCII_dataio (self):
        f = open(self.path_ascii, 'rb').read()
        sio = io.BytesIO(self.ascii_xmld).read()
        self.assertEqual(f, sio)

if __name__ == '__main__':
    unittest.main()



from __future__ import print_function
import pyxb.bundles.opengis.wps_1_0_0 as wps
import pyxb.utils.utility
import sys
import traceback

# Import to define bindings for namespaces that appear in instance documents

for f in sys.argv[1:]:
    print('------------------ %s' % (f,))
    xmld = pyxb.utils.utility.DataFromURI(f)
    try:
        instance = wps.CreateFromDocument(xmld)
        #print xmld
        print(instance.toxml("utf-8"))
    except Exception as e:
        print('%s failed: %s' % (f, e))
        traceback.print_exception(*sys.exc_info())
        sys.exit(1)



# -*- coding: utf-8 -*-
import logging
if __name__ == '__main__':
    logging.basicConfig()
_log = logging.getLogger(__name__)
# Test generated documentation of attributes, elements, and classes
import pyxb.binding.generate
import pyxb.binding.datatypes as xs
import pyxb.binding.basis
import pyxb.utils.domutils
from pyxb.utils import six

import os.path
xsd='''<?xml version="1.0" encoding="UTF-8"?>
<xs:schema xmlns:xs="http://www.w3.org/2001/XMLSchema">
<xs:simpleType name="tEnumerations">
  <xs:annotation><xs:documentation>Documentation for type tEnumerations</xs:documentation></xs:annotation>
  <xs:restriction base="xs:string">
    <xs:enumeration value="one">
      <xs:annotation><xs:documentation>Documentation for tEnumerations.one</xs:documentation></xs:annotation>
    </xs:enumeration>
    <xs:enumeration value="two">
      <xs:annotation><xs:documentation>Documentation for tEnumerations.two</xs:documentation></xs:annotation>
    </xs:enumeration>
  </xs:restriction>
</xs:simpleType>
<xs:complexType name="tComplex">
  <xs:annotation><xs:documentation>Documentation for tComplex</xs:documentation></xs:annotation>
  <xs:complexContent>
    <xs:extension base="xs:anyType">
      <xs:sequence>
        <xs:element name="elt" type="xs:string">
          <xs:annotation><xs:documentation>Documentation for element C{elt} in C{tComplex}</xs:documentation></xs:annotation>
        </xs:element>
        <xs:element ref="element" minOccurs="0" >
          <xs:annotation><xs:documentation>How does documentation for a referenced element come out?</xs:documentation></xs:annotation>
        </xs:element>
      </xs:sequence>
      <xs:attribute name="attr" type="xs:string">
        <xs:annotation><xs:documentation>Documentation for attribute C{attr} in C{tComplex}</xs:documentation></xs:annotation>
      </xs:attribute>
    </xs:extension>
  </xs:complexContent>
</xs:complexType>
<xs:element name="element" type="tComplex">
  <xs:annotation><xs:documentation>Documentation for element C{element}
Multi-line.
With " and ' characters even.
</xs:documentation></xs:annotation>
</xs:element>
</xs:schema>
'''

#open('schema.xsd', 'w').write(xsd)
code = pyxb.binding.generate.GeneratePython(schema_text=xsd)
#open('code.py', 'w').write(code)
#print code

rv = compile(code, 'test', 'exec')
eval(rv)

from pyxb.exceptions_ import *

import unittest

class TestTrac_200908111918 (unittest.TestCase):
    def testComponent (self):
        self.assertEqual(element._description(), '''element (tComplex)
Documentation for element C{element}
Multi-line.
With " and ' characters even.
''')

        desc1 = six.u('''tComplex, element-only content
Attributes:
  attr: attr ({http://www.w3.org/2001/XMLSchema}string), optional
  Wildcard attribute(s)
Elements:
  elt: elt ({http://www.w3.org/2001/XMLSchema}string), local to tComplex
  element: element (tComplex), local to tComplex
  Wildcard element(s)''')
        desc2 = six.u('''tComplex, element-only content
Attributes:
  attr: attr ({http://www.w3.org/2001/XMLSchema}string), optional
  Wildcard attribute(s)
Elements:
  element: element (tComplex), local to tComplex
  elt: elt ({http://www.w3.org/2001/XMLSchema}string), local to tComplex
  Wildcard element(s)''')
        self.assertTrue(tComplex._description() in (desc1, desc2))

        self.assertEqual(tEnumerations._description(), '''tEnumerations restriction of {http://www.w3.org/2001/XMLSchema}string
Documentation for type tEnumerations''')

        # NOTE It is arguably a bug that the local annotation for the
        # reference element has been lost.  When somebody else
        # discovers this and complains, we'll think about fixing it.

        self.assertEqual(tComplex.element.__doc__, '''Documentation for element C{element}
Multi-line.
With " and ' characters even.
''')


if __name__ == '__main__':
    unittest.main()
